local importPath=string.gsub(getfenv(1)._.Name,"%.Arrow","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

--This class creates an "arrow" on screen using a rotated window. The arrow can point between any two xy coordinates (even if they are off screen).
--The arrowhead will be at the second of the two coordinates.
--The width parameter will change the width of the arrow line.
--The headsize parameter is the size of the arrow head in length (width is headsize*2-1)

Arrow=class(Turbine.UI.Window)
function Arrow:Constructor(x1, y1, x2, y2, width, headsize, color, opacity, zorder)
	Turbine.UI.Window.Constructor(self);

	self.X1=x1;
	self.Y1=y1;
	self.X2=x2;
	self.Y2=y2;
	if self.X1==nil then self.X1=0 end;
	if self.Y1==nil then self.Y1=0 end;
	if self.X2==nil then self.X2=0 end;
	if self.Y2==nil then self.Y2=0 end;

	self.Line=Turbine.UI.Control();
	self.Line:SetParent(self);
	self.Line:SetPosition(0,0);

	if color==nil or color.R==nil or color.G==nil or color.B==nil then
		color=Turbine.UI.Color(0,0,0);
	end
	color=Turbine.UI.Color(color.R,color.G,color.B); -- eliminate alpha channel just in case it was provided
	self.Line:SetBackColor(color);

	if opacity==nil then
		self:SetOpacity(1);
	else
		self:SetOpacity(opacity);
	end
	if zorder==nil then
		zorder=0
	end
	self:SetZOrder(zorder);

	self.Rotation={x=0,y=0,z=0};
	self:SetVisible(true);
	self:SetHeight(1); -- height will be the length of the line. we will rotate the window so that the size works
	if width==nil then
		self:SetWidth(1);
		self.Line:SetWidth(1);
	else
		self:SetWidth(width);
		self.Line:SetWidth(width);
	end

	if headsize==nil then
		headsize=3
	else
		headsize=tonumber(headsize)
		if headsize==nil then headsize=3 end
		if headsize<1 then headsize=1 end
		if headsize>7 then headsize=7 end
	end
	self.HeadSize=headsize;

	self.ArrowHead={}
--	local tmpRow, tmpCol;
--	local tmpIndex=1;
--	for tmpRow=1,self.HeadSize do
--		for tmpCol=1,tmpRow*2 do
--			self.ArrowHead[tmpIndex]=Turbine.UI.Control();
--			self.ArrowHead[tmpIndex]:SetParent(self);
--			self.ArrowHead[tmpIndex]:SetSize(1,1);
--			self.ArrowHead[tmpIndex]:SetBackColor(color);
--			tmpIndex=tmpIndex+1;
--		end
--	end
	self.ArrowHead=Turbine.UI.Control()
	self.ArrowHead:SetParent(self)
	self.ArrowHead:SetBackColorBlendMode(Turbine.UI.BlendMode.Color)
	self.ArrowHead:SetSize(self.HeadSize*2-1,self.HeadSize)
	self.ArrowHead:SetBackColor(color)

	self.SetVisible=function(sender,args)
		Turbine.UI.Window.SetVisible(self,args)
		if self:IsVisible() then
			-- when we are displayed we have to reassign the rotation. not sure it this is WAI or a bug in Turbines window class
			Turbine.UI.Window.SetRotation(self,self.Rotation);
		end
	end
	self.SetHeight=function()
		-- don't allow manually setting window height
	end
	self.SetWidth=function(sender,args)
		Turbine.UI.Window.SetWidth(self,args);
		self.Line:SetWidth(args);
	end
	self.SetBackColor=function(sender,args)
		local color
		if args~=nil then
			-- strip out alpha
			color=Turbine.UI.Color(args.R,args.G,args.B)
		end
		self.Line:SetBackColor(color)
		self.ArrowHead:SetBackColor(color)
	end
	self.GetBackColor=function()
		return self.Line:GetBackColor();
	end

	self.SetStartPoint=function(sender,x1, y1)
		if x1~=nil then self.X1=x1 end
		if y1~=nil then self.Y1=y1 end
		self:Draw();
	end

	self.SetEndPoint=function(sender,x1, y1)
		if x1~=nil then self.X2=x1 end
		if y1~=nil then self.Y2=y1 end
		self:Draw();
	end
	self.Draw=function()
		-- use local versions of coordinates so we can swap them if needed
		local x1,y1,x2,y2=self.X1,self.Y1,self.X2,self.Y2;
		local swapped=false;
		if x1>x2 then
			swapped=true;
			-- swap so that the left side of line is always the first coordinates
			local tmpX,tmpY=x2,y2;
			x2=x1
			x1=tmpX
			y2=y1
			y1=tmpY
		end
		-- this does the actual work of the class...
		local x,y=x2-x1,math.abs(y1-y2)
		local theta;
		-- determine the length
		local length=math.sqrt(x*x+y*y);
		self:SetSize(length,length); -- create a window and line of the right dimensions
		self.Line:SetLeft((length-self.Line:GetWidth())/2); -- center line in window so we rotate around the middle of the line
		self.Line:SetHeight(length);
		-- position arrowhead boxes
		if x1==x2 then
			if y1>y2 then
				self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2,0)
				self.ArrowHead:SetBackground(resourcePath.."ArrowheadInv"..tostring(self.HeadSize)..".tga")
			else
				self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2,self:GetHeight()-self.HeadSize+1)
				self.ArrowHead:SetBackground(resourcePath.."Arrowhead"..tostring(self.HeadSize)..".tga")
			end
		else
			if swapped then
				if y1>y2 then
					self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2 ,self:GetHeight()-self.HeadSize+1)
					self.ArrowHead:SetBackground(resourcePath.."Arrowhead"..tostring(self.HeadSize)..".tga")
				else
					self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2,0)
					self.ArrowHead:SetBackground(resourcePath.."ArrowheadInv"..tostring(self.HeadSize)..".tga")
				end
			else
				if y1<=y2 then
					self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2,self:GetHeight()-self.HeadSize+1)
					self.ArrowHead:SetBackground(resourcePath.."Arrowhead"..tostring(self.HeadSize)..".tga")
				else
					-- wrong
					self.ArrowHead:SetPosition(self.Line:GetLeft()-(self.ArrowHead:GetWidth()-self.Line:GetWidth())/2,0)
					self.ArrowHead:SetBackground(resourcePath.."ArrowheadInv"..tostring(self.HeadSize)..".tga")
				end
			end
		end

		if length==0 then
			theta=0;
		else
			theta=math.acos((x1-x2)/length)
		end
		local xoffset,yoffset;
		if y1<=y2 then
			theta=theta-3.1415926535/2
			xoffset=length/2*math.cos(3.1415926535/2-theta)
			yoffset=length/2*math.sin(3.1415926535/2-theta)
		else
			-- adjust to use negative angle
			theta=3.1415926535/2-theta
			xoffset=0-length/2*math.cos(3.1415926535/2-theta)
			yoffset=length/2*math.sin(theta-3.1415926535/2)
		end
		-- the window rotates around center of window, so we adjust so it appears to rotate around the top left corner...
		self:SetPosition(x1-length/2+xoffset,y1-length/2+yoffset)

		-- convert theta to degrees (Turbine's rotation uses degrees while Lua math uses Radians
		self:SetRotation({x=0,y=0,z=theta*180/3.1415926535});

	end
	self:Draw();
end
-- examples:
--arrow1=Arrow(0,0,Turbine.UI.Display:GetWidth(),Turbine.UI.Display:GetHeight()) -- draws a one pixel wide black arrow diagonally across screen
--arrow1=Arrow(0,0,500,500,3,3,Turbine.UI.Color(0,0,1)) -- draws a one pixel wide blue arrow diagonally on screen
--arrow5=Arrow(400,30,200,30,5,8,Turbine.UI.Color(.4,.4,1),.7) --
--arrow6=Arrow(400,30,200,30,3,6,Turbine.UI.Color(0,0,1),.8) --
-- up
--arrow1=Arrow(10,200,10,100,3,6,Turbine.UI.Color(1,0,1),.8) --
-- down
--arrow2=Arrow(20,100,20,200,3,6,Turbine.UI.Color(0,0,1),.8) --
-- right
--arrow3=Arrow(100,10,200,10,3,6,Turbine.UI.Color(1,0,1),.8) --
-- left
--arrow4=Arrow(200,20,100,20,3,6,Turbine.UI.Color(0,0,1),.8) --

--upright
--arrow5=Arrow(100,200,200,100,3,6,Turbine.UI.Color(1,0,1),.8) --
--downleft
--arrow6=Arrow(210,110,110,210,3,6,Turbine.UI.Color(0,0,1),.8) --

--upleft
--arrow7=Arrow(400,200,300,100,3,6,Turbine.UI.Color(1,0,1),.8) --
--downright
--arrow8=Arrow(310,90,410,190,3,6,Turbine.UI.Color(0,0,1),.8) --

--[[
Storyboard:SetZOrder(-1)
Storyboard:SetVisible(true)
if arrow~=nil then
	arrow:SetVisible(false)
	arrow=nil
end



arrow1:SetVisible(false)
arrow2:SetVisible(false)
arrow3:SetVisible(false)
arrow4:SetVisible(false)
arrow5:SetVisible(false)
arrow6:SetVisible(false)
arrow7:SetVisible(false)
arrow8:SetVisible(false)


arrow1=Arrow(200,200,100,200,1,1,Turbine.UI.Color.Red)
arrow2=Arrow(200,200,100,100,1,2,Turbine.UI.Color.Red)
arrow3=Arrow(200,200,200,100,2,3,Turbine.UI.Color.Red)
arrow4=Arrow(200,200,300,100,2,4,Turbine.UI.Color.Red)
arrow5=Arrow(200,200,300,200,8,5,Turbine.UI.Color.Red)
arrow6=Arrow(200,200,300,300,3,6,Turbine.UI.Color.Red)
arrow7=Arrow(200,200,200,300,4,6,Turbine.UI.Color.Red)
arrow8=Arrow(200,200,100,300,4,6,Turbine.UI.Color.Red)


--]]