local importPath=string.gsub(getfenv(1)._.Name,"%.Import","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

pluginName="Anthology"
import (importPath.."Class");
import (importPath.."VindarPatch");
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import (importPath.."Strings");
import (importPath.."Table");
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

displayWidth=Turbine.UI.Display:GetWidth()
displayHeight=Turbine.UI.Display:GetHeight()

if (tonumber("1,000")==1) then
	function euroNormalize(value)
		return tonumber((string.gsub(value,"%.",",")));
	end
else
	function euroNormalize(value)
		return tonumber((string.gsub(value,",",".")));
	end
end

string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end

-- load saved settings - we need this in Main because this supports the OptionsWindow
Settings=PatchDataLoad(Turbine.DataScope.Account, pluginName.."Settings")
if Settings==nil then Settings={} end
if Settings.DataVersion==nil then
	Settings.DataVersion=plugin:GetVersion()
end
if Settings.MaxID==nil then
	Settings.MaxID=0
end
if Settings.Font==nil then
	Settings.Font=Turbine.UI.Lotro.Font.Verdana18
end
if Settings.FontColor==nil then
	Settings.FontColor=Turbine.UI.Color.White
else
	Settings.FontColor=Turbine.UI.Color(euroNormalize(Settings.FontColor.A),euroNormalize(Settings.FontColor.R),euroNormalize(Settings.FontColor.G),euroNormalize(Settings.FontColor.B))
end
if Settings.BackColor==nil then
	Settings.BackColor=Turbine.UI.Color.Black
else
	Settings.BackColor=Turbine.UI.Color(euroNormalize(Settings.BackColor.A),euroNormalize(Settings.BackColor.R),euroNormalize(Settings.BackColor.G),euroNormalize(Settings.BackColor.B))
end
if Settings.HeadingColor==nil then
	Settings.HeadingColor=Turbine.UI.Color.LightBlue
else
	Settings.HeadingColor=Turbine.UI.Color(euroNormalize(Settings.HeadingColor.A),euroNormalize(Settings.HeadingColor.R),euroNormalize(Settings.HeadingColor.G),euroNormalize(Settings.HeadingColor.B))
end
if Settings.LabelColor==nil then
	Settings.LabelColor=Turbine.UI.Color(.843,.698,.227)
else
	Settings.LabelColor=Turbine.UI.Color(euroNormalize(Settings.LabelColor.A),euroNormalize(Settings.LabelColor.R),euroNormalize(Settings.LabelColor.G),euroNormalize(Settings.LabelColor.B))
end
if Settings.TabSelectedColor==nil then
	Settings.TabSelectedColor=Turbine.UI.Color.LightBlue
else
	-- we have to euroNormalize the colors in case the calling plugin doesn't support font colors
	Settings.TabSelectedColor=Turbine.UI.Color(euroNormalize(Settings.TabSelectedColor.A),euroNormalize(Settings.TabSelectedColor.R),euroNormalize(Settings.TabSelectedColor.G),euroNormalize(Settings.TabSelectedColor.B))
end
if Settings.TabUnSelectedColor==nil then
	Settings.TabUnSelectedColor=Turbine.UI.Color(.2,.25,1)
else
	-- we have to euroNormalize the colors in case the calling plugin doesn't support font colors
	Settings.TabUnSelectedColor=Turbine.UI.Color(euroNormalize(Settings.TabUnSelectedColor.A),euroNormalize(Settings.TabUnSelectedColor.R),euroNormalize(Settings.TabUnSelectedColor.G),euroNormalize(Settings.TabUnSelectedColor.B))
end

if Settings.TrimColor==nil then
	Settings.TrimColor=Turbine.UI.Color(.15,.15,.25)
else
	Settings.TrimColor=Turbine.UI.Color(euroNormalize(Settings.TrimColor.A),euroNormalize(Settings.TrimColor.R),euroNormalize(Settings.TrimColor.G),euroNormalize(Settings.TrimColor.B))
end

if Settings.LanguageEN==nil then
	Settings.LanguageEN=(locale == "en")
end
if Settings.LanguageDE==nil then
	Settings.LanguageDE=(locale == "de")
end
if Settings.LanguageFR==nil then
	Settings.LanguageFR=(locale == "fr")
end
if Settings.Opacity==nil then
	Settings.Opacity=1
else
	Settings.Opacity=euroNormalize(Settings.Opacity)
end

StoryList=PatchDataLoad(Turbine.DataScope.Account, pluginName.."StoryList");
if StoryList==nil then StoryList={} end
for k,v in ipairs(StoryList) do
	if v.ID==nil then
		v.ID=Settings.MaxID+1
		Settings.MaxID=v.ID
	end
	if v.ID>Settings.MaxID then Settings.MaxID=v.ID end
end
import (importPath.."DropDownList");
import (importPath.."FontSupport")
fontMetric=FontMetric()
fontMetric:SetFont(Settings.Font)
import (importPath.."DebugWindow");
import (importPath.."PopUpDialog");
import (importPath.."IconWindow")
import (importPath.."Settings")
import (importPath.."StorySelect") -- used for Export

getStoryIndexFromID=function(id)
	local index
	for k,v in ipairs(StoryList) do
		if v.ID==id then
			index=k
			break
		end
	end
	return index
end
getNewID=function()
	-- this function will return the first unused ID for a new story. if they are all used sequentially then it returns the next ID
	-- build table of IDs
	local ret=1
	local tmpList={}
	for k,v in ipairs(StoryList) do
		table.insert(tmpList,v.ID)
	end
	if #tmpList>0 then
		-- sort table of IDs
		table.sort(tmpList)
		if tmpList[1]==1 then
			local found=false
			-- find first entry that doesn't match index
			for k,v in ipairs(tmpList) do
				if k~=v then
					ret=k
					found=true
					break
				end
			end
			if not found then
				ret=#tmpList+1
			end
		end
	end
	return ret
end
verifyFileName=function(fileName)
	local ret=true
	for i,v in ipairs({fileName:byte(1,-1)}) do
		if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
			ret = false
			break
		end
	end
	return ret
end

importStory={}
importStory.Chapters={}

importWindow=Turbine.UI.Lotro.Window()
importWindow:SetResizable(true)
importWindow:SetMinimumSize(700,350)

if Settings.ImportWindow==nil then
	Settings.ImportWindow={}
end
if Settings.ImportWindow.Width==nil then
	Settings.ImportWindow.Width=.67
else
	Settings.ImportWindow.Width=euroNormalize(Settings.ImportWindow.Width)
end
if Settings.ImportWindow.Height==nil then
	Settings.ImportWindow.Height=.5
else
	Settings.ImportWindow.Height=euroNormalize(Settings.ImportWindow.Height)
end
if Settings.ImportWindow.Top==nil then
	Settings.ImportWindow.Top=(1-Settings.ImportWindow.Height)/2
else
	Settings.ImportWindow.Top=euroNormalize(Settings.ImportWindow.Top)
end
if Settings.ImportWindow.Left==nil then
	Settings.ImportWindow.Left=(1-Settings.ImportWindow.Height)/2
else
	Settings.ImportWindow.Left=euroNormalize(Settings.ImportWindow.Left)
end
if Settings.ImportWindow.Width<(importWindow:GetMinimumWidth()/displayWidth) then Settings.ImportWindow.Width=importWindow:GetMinimumWidth()/displayWidth end
if Settings.ImportWindow.Height<(importWindow:GetMinimumHeight()/displayHeight) then Settings.ImportWindow.Height=importWindow:GetMinimumHeight()/displayWidth end
loadMenu=function()
	Turbine.PluginManager.LoadPlugin("Anthology")
	importWindow:SetVisible(false)
end
launchTutorial=function()
	SaveSetting("Account",pluginName.."Settings","","ReaderStoryID",Settings.TutorialID) -- save story id
	SaveSetting("Account",pluginName.."Settings","","Mode","Reader") -- trigger reload, delayed until AFTER this plugin unloads
	SettingQueueCallback(loadMenu)
end
if Settings.ProcessTutorial~=nil and Settings.ProcessTutorial and _G.importStory~=nil then
	Settings.Mode=""
	-- need to apply inverse VindarPatch
	local success,tempStory=pcall(convBack,_G.importStory());
	if _G.imageRoot~=nil and tempStory.DefaultImagePath==nil then tempStory.DefaultImagePath=_G.imageRoot end

	local ID=getNewID()
	local Author, Name, Desc
	if tempStory.Author==nil then
		Author=""
	else
		Author=tostring(tempStory.Author)
	end
	if tempStory.Name==nil then
		Name=Settings.ImportName
	else
		Name=tostring(tempStory.Name)
	end
	if tempStory.Desc==nil then
		Desc=""
	else
		Desc=tostring(tempStory.Desc)
	end
	for k,v in ipairs(StoryList) do
		if v.Name==Name and v.Author==Author then
			Name=Name.." - "..Resource[language][47]
			tempStory.Name=Name
			errorMsg=Resource[language][181]
			break
		end
	end
	if ID>Settings.MaxID then Settings.MaxID=ID end
	table.insert(StoryList,{["ID"]=ID,["Author"]=Author,["Name"]=Name,["Desc"]=Desc,["LuaName"]="AnthologyTutorial"})
	PatchDataSave(Turbine.DataScope.Account, pluginName.."Story"..tostring(ID),tempStory)
	PatchDataSave(Turbine.DataScope.Account, pluginName.."StoryList",StoryList)
	if errorMsg==nil then		
		Settings.TutorialID=ID
		SettingQueueCallback(launchTutorial)
	end
end
if Settings.Mode=="ImportDat" then
	-- try to read the story
	tempStory=PatchDataLoad(Turbine.DataScope.Account, Settings.ImportName)
	if tempStory==nil then
		errorMsg=Resource[language][180]
	else
		-- we found a file. not sure it's a story but we will save it as one and see what happens...
		local ID=getNewID()
		local Author, Name, Desc
		if tempStory.Author==nil then
			Author=""
		else
			Author=tostring(tempStory.Author)
		end
		if tempStory.Name==nil then
			Name=Settings.ImportName
		else
			Name=tostring(tempStory.Name)
		end
		if tempStory.Desc==nil then
			Desc=""
		else
			Desc=tostring(tempStory.Desc)
		end
		for k,v in ipairs(StoryList) do
			if v.Name==Name and v.Author==Author then
				Name=Name.." - "..Resource[language][47]
				tempStory.Name=Name
				errorMsg=Resource[language][181]
				break
			end
		end
		if ID>Settings.MaxID then Settings.MaxID=ID end
		table.insert(StoryList,{["ID"]=ID,["Author"]=Author,["Name"]=Name,["Desc"]=Desc})
		PatchDataSave(Turbine.DataScope.Account, pluginName.."Story"..tostring(ID),tempStory)
		if errorMsg==nil then		
			Settings.ImportStoryID=ID
		end
	end
elseif Settings.Mode=="Export" then
	local currentStoryID=Settings.ImportStoryID
	-- load the file into tempStory and then immediately save it to the specified name
	tempStory=PatchDataLoad(Turbine.DataScope.Account, pluginName.."Story"..tostring(currentStoryID))
	PatchDataSave(Turbine.DataScope.Account, Settings.ExportName, tempStory)
end

importWindow:SetSize(Settings.ImportWindow.Width*displayWidth,Settings.ImportWindow.Height*displayHeight)
importWindow:SetPosition(Settings.ImportWindow.Left*displayWidth,Settings.ImportWindow.Top*displayHeight)

importWindow:SetText(Resource[language][2].." - "..Resource[language][6])

importWindow.ImportPluginHeading=Turbine.UI.Label()
importWindow.ImportPluginHeading:SetParent(importWindow)
importWindow.ImportPluginHeading:SetPosition(10,35)
importWindow.ImportPluginHeading:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
importWindow.ImportPluginCaption=Turbine.UI.Label()
importWindow.ImportPluginCaption:SetParent(importWindow)
importWindow.ImportPluginCaption:SetPosition(0,0)
importWindow.ImportPlugin=DropDownList()
importWindow.ImportPlugin:SetParent(importWindow)
importWindow.ImportPlugin:SetZOrder(1)
-- load available plugins with a package in Garan.Anthology.Stories.x
pluginTable=Turbine.PluginManager.GetAvailablePlugins();
importWindow.ImportPlugin:AddItem(Resource[language][96],0)
for k,v in ipairs(pluginTable) do
	if string.sub(v.Package,1,29)=="GaranStuff.Anthology.Stories." then
		-- make sure same story doesn't already exist in StoryList
		local found=false
		for index,story in ipairs(StoryList) do
			if story.Author==v.Author and story.LuaName~=nil and story.LuaName==v.Name then
				found=true
				break
			end
		end
		if not found then
			importWindow.ImportPlugin:AddItem(v.Name,k)
		end
	end
end
importWindow.AuthorCaption=Turbine.UI.Label()
importWindow.AuthorCaption:SetParent(importWindow)
importWindow.Author=Turbine.UI.Label()
importWindow.Author:SetParent(importWindow)
importWindow.Author:SetMultiline(false)

importWindow.DescriptionCaption=Turbine.UI.Label()
importWindow.DescriptionCaption:SetParent(importWindow)
importWindow.Description=Turbine.UI.Label()
importWindow.Description:SetParent(importWindow)
importWindow.DescriptionVScroll=Turbine.UI.Lotro.ScrollBar()
importWindow.DescriptionVScroll:SetParent(importWindow)
importWindow.DescriptionVScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
importWindow.DescriptionVScroll:SetWidth(10)
importWindow.Description:SetVerticalScrollBar(importWindow.DescriptionVScroll)

importWindow.ImportLua=Turbine.UI.Lotro.Button()
importWindow.ImportLua:SetParent(importWindow)
importWindow.ImportLua:SetEnabled(false)
importWindow.ImportLua.Click=function()
	_G.importStory=nil
	-- import the .lua file by loading the plugin
	local success,msg=pcall(Turbine.PluginManager.LoadPlugin,pluginTable[importWindow.ImportPlugin:GetValue()].Name)
	if not success then
		-- generate a crit popup
		importWindow.MouseMask:SetMouseVisible(true)
		popUp=PopUpDialog(Resource[language][112],Resource[language][169]..msg,1,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
	else
		if _G.importStory==nil then
			popUp=PopUpDialog(Resource[language][112],Resource[language][170],1,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
		else
			importStory=_G.importStory()
			if importStory==nil then
				popUp=PopUpDialog(Resource[language][112],Resource[language][170],1,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
			else
				-- apply inverse Vindar patch
				success,importStory=pcall(convBack,importStory);
				if success then
					local found=false
					for k,v in ipairs(StoryList) do
						if v.Name==importStory.Name and v.Author==importStory.Author then
							found=true
							importStory.Name=importStory.Name.." - "..Resource[language][47]
							popUp=PopUpDialog(Resource[language][39],Resource[language][40],1,Resource[language][32],Resource[language][19],nil,false,confirmImport)
							break
						end
					end
					if not found then
						confirmImport(1)
					end
				else
					popUp=PopUpDialog(Resource[language][112],Resource[language][174],1,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
				end

			end
		end
	end
end
confirmImport=function(flag)
	popUp=nil
	if flag==1 then
		importWindow.currentStoryID=getNewID()
		if importWindow.currentStoryID>Settings.MaxID then
			Settings.MaxID=importWindow.currentStoryID
		end
		table.insert(StoryList,{["ID"]=importWindow.currentStoryID,["Author"]=importStory.Author,["LuaName"]=pluginTable[importWindow.ImportPlugin:GetValue()].Name,["Name"]=importStory.Name,["Desc"]=importStory.Desc})
		SaveSetting("Account",pluginName.."Settings","","MaxID",Settings.MaxID)
		SaveSetting("Account",pluginName.."Settings","","ImportStoryID",importWindow.currentStoryID)
		SaveSetting("Account",pluginName.."Settings","","Mode","ImportLua") -- trigger reload
		Settings.Mode="ImportLua"
		-- need to save import settings
		SaveSettings()
		SettingQueueCallback(loadMenu)
	end
end
importWindow.ClosePopUp=function()
	popUp=nil
	importWindow.MouseMask:SetMouseVisible(false)
end
importWindow.ImportPlugin.SelectedIndexChanged=function()
	local index=importWindow.ImportPlugin:GetValue()
	if index>0 then
		importWindow.Author:SetText(pluginTable[index].Author)
		importWindow.Description:SetText(pluginTable[index].Description)
		importWindow.ImportLua:SetEnabled(true)
	else
		importWindow.Author:SetText("")
		importWindow.Description:SetText("")
		importWindow.ImportLua:SetEnabled(false)
	end
end

importWindow.Split=Turbine.UI.Control()
importWindow.Split:SetParent(importWindow)
importWindow.Split:SetHeight(3)
importWindow.SplitCaption=Turbine.UI.Label()
importWindow.SplitCaption:SetParent(importWindow)
importWindow.SplitCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
importWindow.Split2=Turbine.UI.Control()
importWindow.Split2:SetParent(importWindow)
importWindow.Split2:SetHeight(3)

importWindow.ImportPlugindataHeading=Turbine.UI.Label()
importWindow.ImportPlugindataHeading:SetParent(importWindow)
importWindow.ImportPlugindataHeading:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
importWindow.ImportPlugindataCaption=Turbine.UI.Label()
importWindow.ImportPlugindataCaption:SetParent(importWindow)
importWindow.ImportPlugindata=Turbine.UI.Lotro.TextBox()
importWindow.ImportPlugindata:SetParent(importWindow)
importWindow.ImportPlugindata:SetMultiline(false)
if errorMsg~=nil then
	importWindow.ImportPlugindata:SetText(Settings.ImportName)
end
importWindow.ImportPlugindata.TextChanged=function()
	importWindow.ImportDat:SetEnabled(importWindow.ImportPlugindata:GetText()~="")
end

importWindow.ImportDat=Turbine.UI.Lotro.Button()
importWindow.ImportDat:SetParent(importWindow)
importWindow.ImportDat:SetEnabled(false)
importWindow.ImportDat.Click=function()
	if verifyFileName(importWindow.ImportPlugindata:GetText()) then
		-- need to reload and import the file
		SaveSetting("Account",pluginName.."Settings","","ImportName",importWindow.ImportPlugindata:GetText())
		SaveSetting("Account",pluginName.."Settings","","Mode","ImportDat") -- trigger reload
		Settings.Mode="ImportDat"
		-- need to save import settings
		SaveSettings()
		SettingQueueCallback(loadMenu)
	else
		popUp=PopUpDialog(Resource[language][181],Resource[language][182],2,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
	end
end

importWindow.Split3=Turbine.UI.Control()
importWindow.Split3:SetParent(importWindow)
importWindow.Split3:SetHeight(3)
importWindow.SplitCaption2=Turbine.UI.Label()
importWindow.SplitCaption2:SetParent(importWindow)
importWindow.SplitCaption2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
importWindow.Split4=Turbine.UI.Control()
importWindow.Split4:SetParent(importWindow)
importWindow.Split4:SetHeight(3)

importWindow.ExportHeading=Turbine.UI.Label()
importWindow.ExportHeading:SetParent(importWindow)
importWindow.ExportHeading:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)

importWindow.ExportStoryCaption=Turbine.UI.Label()
importWindow.ExportStoryCaption:SetParent(importWindow)
importWindow.ExportStory=Turbine.UI.Lotro.TextBox()
importWindow.ExportStory:SetParent(importWindow)
importWindow.ExportStory:SetMultiline(false)
importWindow.ExportStory.FocusGained=function()
	importWindow.ExportStorySelect:Focus()
end
importWindow.ExportStoryMask=Turbine.UI.Control()
importWindow.ExportStoryMask:SetParent(importWindow)
importWindow.ExportStoryMask.MouseClick=function()
	importWindow.ExportStorySelect.Click()
end
importWindow.ExportStorySelect=Turbine.UI.Button()
importWindow.ExportStorySelect:SetParent(importWindow)
importWindow.ExportStorySelect:SetText("...")
importWindow.ExportStorySelect.Click=function()
	storySelect.currentStoryID=importWindow.currentStoryID
	storySelect.ReturnSelected=importWindow.StorySelected
	storySelect.FillStoryList(storySelect.StoryListBox.SortColumn) -- refresh story list in case a story has been started or stopped
	storySelect:SetVisible(true)
	importWindow.MouseMask:SetMouseVisible(true) -- note, MouseMask is used to simulate modal dialogs
end
importWindow.StorySelected=function()
	importWindow.MouseMask:SetMouseVisible(false)
	importWindow.currentStoryID=storySelect.SelectedID
	if importWindow.currentStoryID~=nil then
		index=getStoryIndexFromID(storySelect.SelectedID)
		importWindow.ExportStory:SetText(StoryList[index].Name)
		importWindow.ExportButton:SetEnabled(importWindow.ExportPlugindata:GetText()~="")
	end
end
importWindow.ExportPlugindataCaption=Turbine.UI.Label()
importWindow.ExportPlugindataCaption:SetParent(importWindow)
importWindow.ExportPlugindata=Turbine.UI.Lotro.TextBox()
importWindow.ExportPlugindata:SetParent(importWindow)
importWindow.ExportPlugindata:SetMultiline(false)
importWindow.ExportPlugindata.TextChanged=function()
	importWindow.ExportButton:SetEnabled(importWindow.currentStoryID~=nil and importWindow.ExportPlugindata:GetText()~="")
end

importWindow.ExportButton=Turbine.UI.Lotro.Button()
importWindow.ExportButton:SetParent(importWindow)
importWindow.ExportButton:SetEnabled(false)
importWindow.ExportButton.Click=function()
	if importWindow.currentStoryID~=nil and importWindow.ExportPlugindata:GetText()~="" then
		if verifyFileName(importWindow.ExportPlugindata:GetText()) then
			-- reload and process
			SaveSetting("Account",pluginName.."Settings","","ImportStoryID",importWindow.currentStoryID)
			SaveSetting("Account",pluginName.."Settings","","ExportName",importWindow.ExportPlugindata:GetText())
			SaveSetting("Account",pluginName.."Settings","","Mode","Export") -- trigger reload
			SaveSettings()
			SettingQueueCallback(loadMenu)
		else
			popUp=PopUpDialog(Resource[language][181],Resource[language][182],2,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)
		end
	end
end

importWindow.MenuButton=Turbine.UI.Lotro.Button()
importWindow.MenuButton:SetParent(importWindow)
importWindow.MenuButton.Click=function()
	SaveSetting("Account",pluginName.."Settings","","ImportStoryID",nil)
	SaveSetting("Account",pluginName.."Settings","","Mode","Menu") -- trigger reload
	-- need to save import settings
	SaveSettings()
	SettingQueueCallback(loadMenu)
end

importWindow.MouseMask=Turbine.UI.Control()
importWindow.MouseMask:SetParent(importWindow)
importWindow.MouseMask:SetMouseVisible(false)
importWindow.MouseMask:SetZOrder(2)
importWindow.MouseMask:SetSize(importWindow:GetSize())

importWindow.SizeChanged=function(sender,args)
	importWindow.RefreshLayout()
end
SaveSettings=function()
	SaveSetting("Account",pluginName.."Settings","ImportWindow","Width",importWindow:GetWidth()/displayWidth)
	SaveSetting("Account",pluginName.."Settings","ImportWindow","Height",importWindow:GetHeight()/displayHeight)
	SaveSetting("Account",pluginName.."Settings","ImportWindow","Top",importWindow:GetTop()/displayHeight)
	SaveSetting("Account",pluginName.."Settings","ImportWindow","Left",importWindow:GetLeft()/displayWidth)
	SaveSetting("Account",pluginName.."Settings","IconWindow","Left",iconWindow:GetLeft()/displayWidth)
	SaveSetting("Account",pluginName.."Settings","IconWindow","Top",iconWindow:GetTop()/displayHeight)
end

importWindow.RefreshLayout=function()
	local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
	local buttonFont=Settings.Font
	while Turbine.UI.Lotro.FontInfo[buttonFont].size>22 do
		-- make sure button text fits on buttons since they don't size larger than 20 pixels (22 is ok since lotro font sizes are off a bit)
		buttonFont=GetSmallerFont(buttonFont)
	end

	fontMetric:SetFont(buttonFont)
	local minButtonWidth=140
	local tmp=fontMetric:GetTextWidth(Resource[language][7])
	if tmp>minButtonWidth then minButtonWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][167])
	if tmp>minButtonWidth then minButtonWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][168])
	if tmp>minButtonWidth then minButtonWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][177])
	if tmp>minButtonWidth then minButtonWidth=tmp end

	fontMetric:SetFont(Settings.Font)
	local labelWidth=fontMetric:GetTextWidth(Resource[language][163]..":")+20
	local tmp=fontMetric:GetTextWidth(Resource[language][12]..":")+20
	if tmp>labelWidth then labelWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][15]..":")+20
	if tmp>labelWidth then labelWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][165]..":")+20
	if tmp>labelWidth then labelWidth=tmp end
	local tmp=fontMetric:GetTextWidth(Resource[language][176]..":")+20
	if tmp>labelWidth then labelWidth=tmp end

	importWindow.ImportPluginHeading:SetFont(Settings.Font)
	importWindow.ImportPluginHeading:SetText(Resource[language][162])
	importWindow.ImportPluginHeading:SetSize(importWindow:GetWidth()-20,controlHeight)

	importWindow.ImportPluginCaption:SetFont(Settings.Font)
	importWindow.ImportPluginCaption:SetPosition(10,importWindow.ImportPluginHeading:GetTop()+importWindow.ImportPluginHeading:GetHeight()+10)
	importWindow.ImportPluginCaption:SetSize(labelWidth,controlHeight)
	importWindow.ImportPluginCaption:SetText(Resource[language][163]..":")
	importWindow.ImportPlugin:SetPosition(importWindow.ImportPluginCaption:GetLeft()+importWindow.ImportPluginCaption:GetWidth()+5,importWindow.ImportPluginCaption:GetTop())
	importWindow.ImportPlugin:SetSize(importWindow:GetWidth()-importWindow.ImportPlugin:GetLeft()-10)
	importWindow.ImportPlugin:SetFont(Settings.Font)

	importWindow.AuthorCaption:SetPosition(importWindow.ImportPluginCaption:GetLeft(),importWindow.ImportPluginCaption:GetTop()+importWindow.ImportPluginCaption:GetHeight()+5)
	importWindow.AuthorCaption:SetSize(labelWidth,controlHeight)
	importWindow.AuthorCaption:SetFont(Settings.Font)
	importWindow.AuthorCaption:SetText(Resource[language][12]..":")
	importWindow.Author:SetPosition(importWindow.ImportPlugin:GetLeft(),importWindow.AuthorCaption:GetTop())
	importWindow.Author:SetSize(importWindow:GetWidth()-importWindow.Author:GetLeft()-10,controlHeight)
	importWindow.Author:SetFont(Settings.Font)
	importWindow.Author:SetText(importWindow.Author:GetText())

	-- everthing from here is positioned in reverse, relative to bottom of window to allow Description to expand with window

	importWindow.MenuButton:SetWidth(minButtonWidth)
	importWindow.MenuButton:SetPosition((importWindow:GetWidth()-importWindow.MenuButton:GetWidth())/2,importWindow:GetHeight()-35)
	importWindow.MenuButton:SetFont(buttonFont)
	importWindow.MenuButton:SetText(Resource[language][7])

	importWindow.ExportButton:SetWidth(minButtonWidth)
	importWindow.ExportButton:SetPosition(importWindow.MenuButton:GetLeft(),importWindow.MenuButton:GetTop()-25)
	importWindow.ExportButton:SetFont(buttonFont)
	importWindow.ExportButton:SetText(Resource[language][177])

	importWindow.ExportPlugindataCaption:SetSize(labelWidth,controlHeight)
	importWindow.ExportPlugindataCaption:SetPosition(10,importWindow.ExportButton:GetTop()-5-controlHeight)
	importWindow.ExportPlugindataCaption:SetFont(Settings.Font)
	importWindow.ExportPlugindataCaption:SetText(Resource[language][178]..":")

	importWindow.ExportPlugindata:SetPosition(importWindow.Author:GetLeft(),importWindow.ExportPlugindataCaption:GetTop())
	importWindow.ExportPlugindata:SetSize(importWindow.Author:GetWidth(),controlHeight)
	importWindow.ExportPlugindata:SetFont(Settings.Font)
	importWindow.ExportPlugindata:SetText(importWindow.ExportPlugindata:GetText())

	importWindow.ExportStoryCaption:SetSize(labelWidth,controlHeight)
	importWindow.ExportStoryCaption:SetPosition(10,importWindow.ExportPlugindataCaption:GetTop()-5-controlHeight)
	importWindow.ExportStoryCaption:SetFont(Settings.Font)
	importWindow.ExportStoryCaption:SetText(Resource[language][176]..":")
	importWindow.ExportStorySelect:SetSize(fontMetric:GetTextWidth("..."),controlHeight)
	importWindow.ExportStorySelect:SetPosition(importWindow:GetWidth()-10-importWindow.ExportStorySelect:GetWidth(),importWindow.ExportStoryCaption:GetTop())
	importWindow.ExportStorySelect:SetFont(Settings.Font)
	importWindow.ExportStorySelect:SetText("...")
	importWindow.ExportStory:SetPosition(importWindow.Author:GetLeft(),importWindow.ExportStoryCaption:GetTop())
	importWindow.ExportStory:SetSize(importWindow.Author:GetWidth()-importWindow.ExportStorySelect:GetWidth(),controlHeight)
	importWindow.ExportStory:SetFont(Settings.Font)
	importWindow.ExportStory:SetText(importWindow.ExportStory:GetText())
	importWindow.ExportStoryMask:SetSize(importWindow.ExportStory:GetWidth(),importWindow.ExportStory:GetHeight())
	importWindow.ExportStoryMask:SetPosition(importWindow.ExportStory:GetLeft(),importWindow.ExportStory:GetTop())

	importWindow.ExportHeading:SetSize(importWindow:GetWidth()-20,controlHeight)
	importWindow.ExportHeading:SetPosition(10,importWindow.ExportStoryCaption:GetTop()-controlHeight-5)
	importWindow.ExportHeading:SetFont(Settings.Font)
	importWindow.ExportHeading:SetText(Resource[language][175])

	importWindow.SplitCaption2:SetFont(Settings.Font)
	importWindow.SplitCaption2:SetSize(fontMetric:GetTextWidth(Resource[language][166])+10,controlHeight)
	importWindow.SplitCaption2:SetText(Resource[language][166])
	importWindow.SplitCaption2:SetPosition((importWindow:GetWidth()-importWindow.SplitCaption2:GetWidth())/2,importWindow.ExportHeading:GetTop()-5-importWindow.SplitCaption2:GetHeight())
	importWindow.Split3:SetPosition(5,importWindow.SplitCaption2:GetTop()+importWindow.SplitCaption2:GetHeight()/2-1)
	importWindow.Split3:SetWidth((importWindow:GetWidth()-10-importWindow.SplitCaption2:GetWidth())/2)
	importWindow.Split4:SetPosition(importWindow.SplitCaption2:GetLeft()+importWindow.SplitCaption2:GetWidth(),importWindow.Split3:GetTop())
	importWindow.Split4:SetWidth(importWindow.Split3:GetWidth())

	importWindow.ImportDat:SetWidth(minButtonWidth)
	importWindow.ImportDat:SetFont(buttonFont)
	importWindow.ImportDat:SetText(Resource[language][168])
	importWindow.ImportDat:SetPosition(importWindow.MenuButton:GetLeft(),importWindow.SplitCaption2:GetTop()-25)

	importWindow.ImportPlugindataCaption:SetSize(labelWidth,controlHeight)
	importWindow.ImportPlugindataCaption:SetPosition(10,importWindow.ImportDat:GetTop()-5-controlHeight)
	importWindow.ImportPlugindataCaption:SetFont(Settings.Font)
	importWindow.ImportPlugindataCaption:SetText(Resource[language][165]..":")

	importWindow.ImportPlugindata:SetPosition(importWindow.Author:GetLeft(),importWindow.ImportPlugindataCaption:GetTop())
	importWindow.ImportPlugindata:SetSize(importWindow.Author:GetWidth(),controlHeight)
	importWindow.ImportPlugindata:SetFont(Settings.Font)
	importWindow.ImportPlugindata:SetText(importWindow.ImportPlugindata:GetText())

	importWindow.ImportPlugindataHeading:SetSize(importWindow:GetWidth()-20,controlHeight)
	importWindow.ImportPlugindataHeading:SetPosition(10,importWindow.ImportPlugindataCaption:GetTop()-controlHeight-5)
	importWindow.ImportPlugindataHeading:SetFont(Settings.Font)
	importWindow.ImportPlugindataHeading:SetText(Resource[language][164])

	importWindow.SplitCaption:SetFont(Settings.Font)
	importWindow.SplitCaption:SetSize(fontMetric:GetTextWidth(Resource[language][166])+10,controlHeight)
	importWindow.SplitCaption:SetText(Resource[language][166])
	importWindow.SplitCaption:SetPosition((importWindow:GetWidth()-importWindow.SplitCaption:GetWidth())/2,importWindow.ImportPlugindataHeading:GetTop()-5-importWindow.SplitCaption:GetHeight())
	importWindow.Split:SetPosition(5,importWindow.SplitCaption:GetTop()+importWindow.SplitCaption:GetHeight()/2-1)
	importWindow.Split:SetWidth((importWindow:GetWidth()-10-importWindow.SplitCaption:GetWidth())/2)
	importWindow.Split2:SetPosition(importWindow.SplitCaption:GetLeft()+importWindow.SplitCaption:GetWidth(),importWindow.Split:GetTop())
	importWindow.Split2:SetWidth(importWindow.Split:GetWidth())

	importWindow.ImportLua:SetWidth(minButtonWidth)
	importWindow.ImportLua:SetFont(buttonFont)
	importWindow.ImportLua:SetText(Resource[language][167])
	importWindow.ImportLua:SetPosition(importWindow.MenuButton:GetLeft(),importWindow.SplitCaption:GetTop()-controlHeight-5)

	importWindow.DescriptionCaption:SetPosition(importWindow.ImportPluginCaption:GetLeft(),importWindow.AuthorCaption:GetTop()+importWindow.AuthorCaption:GetHeight()+5)
	importWindow.DescriptionCaption:SetSize(labelWidth,importWindow.ImportLua:GetTop()-importWindow.DescriptionCaption:GetTop()-5)
	importWindow.DescriptionCaption:SetFont(Settings.Font)
	importWindow.DescriptionCaption:SetText(Resource[language][15]..":")
	importWindow.Description:SetPosition(importWindow.ImportPlugin:GetLeft(),importWindow.DescriptionCaption:GetTop())
	importWindow.Description:SetSize(importWindow:GetWidth()-importWindow.Description:GetLeft()-20,controlHeight*3)
	importWindow.Description:SetFont(Settings.Font)
	importWindow.Description:SetText(importWindow.Description:GetText())
	importWindow.DescriptionVScroll:SetPosition(importWindow.Description:GetLeft()+importWindow.Description:GetWidth(),importWindow.Description:GetTop())
	importWindow.DescriptionVScroll:SetHeight(importWindow.Description:GetHeight())

	importWindow.MouseMask:SetSize(importWindow:GetSize())
end

importWindow.RefreshLayout()
importWindow.RefreshColor=function()
	importWindow.ImportPluginHeading:SetForeColor(Settings.HeadingColor)
	importWindow.ImportPluginCaption:SetForeColor(Settings.LabelColor)
	importWindow.ImportPlugin:SetBorderColor(Settings.TrimColor)
	importWindow.ImportPlugin:SetBackColor(Settings.BackColor)
	importWindow.ImportPlugin.CurrentValue:SetBackColor(Settings.BackColor)
	importWindow.ImportPlugin:SetTextColor(Settings.FontColor)
	importWindow.AuthorCaption:SetForeColor(Settings.LabelColor)
	importWindow.Author:SetForeColor(Settings.FontColor)
	importWindow.DescriptionCaption:SetForeColor(Settings.LabelColor)
	importWindow.Description:SetForeColor(Settings.FontColor)
	importWindow.Split:SetBackColor(Settings.TrimColor)
	importWindow.SplitCaption:SetForeColor(Settings.LabelColor)
	importWindow.Split2:SetBackColor(Settings.TrimColor)
	importWindow.ImportPlugindataHeading:SetForeColor(Settings.HeadingColor)
	importWindow.ImportPlugindataCaption:SetForeColor(Settings.LabelColor)
	importWindow.ImportPlugindata:SetForeColor(Settings.FontColor)
	importWindow.Split3:SetBackColor(Settings.TrimColor)
	importWindow.SplitCaption2:SetForeColor(Settings.LabelColor)
	importWindow.Split4:SetBackColor(Settings.TrimColor)
	importWindow.ExportHeading:SetForeColor(Settings.HeadingColor)
	importWindow.ExportStoryCaption:SetForeColor(Settings.LabelColor)
	importWindow.ExportPlugindataCaption:SetForeColor(Settings.LabelColor)
	importWindow.ExportStory:SetForeColor(Settings.FontColor)
	importWindow.ExportPlugindata:SetForeColor(Settings.FontColor)
end
importWindow.RefreshColor()
importWindow.StoryDeleted=function(ID)
end
importWindow.Closed=function()
	iconWindow:SetVisible(true)
end
if Settings.TutorialID==nil then
	importWindow:SetVisible(true)
end
currentWindow=importWindow
-- show success message if prior import
if Settings.ImportStoryID~=nil then
	if Settings.Mode=="ImportLua" or Settings.Mode=="ImportDat" then
		importWindow.MouseMask:SetMouseVisible(true)
		popUp=PopUpDialog(Resource[language][172],Resource[language][173],4,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)	
	elseif Settings.Mode=="Export" then
		importWindow.MouseMask:SetMouseVisible(true)
		popUp=PopUpDialog(Resource[language][172],Resource[language][179],4,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)	
	end
	Settings.ImportStoryID=nil
	SaveSetting("Account",pluginName.."Settings","","ImportStoryID",nil)
end
if errorMsg~=nil then
	importWindow.MouseMask:SetMouseVisible(true)
	popUp=PopUpDialog(Resource[language][112],tostring(errorMsg),4,Resource[language][32],nil,nil,false,importWindow.ClosePopUp)	
	errorMsg=nil
end

unloadMain=function()
	Settings.Mode=""
	success,result=pcall(Turbine.PluginManager.UnloadScriptState,"Anthology");
end
SettingQueueCallback(unloadMain)
plugin.Unload=function()
	if importWindow.currentStoryID~=nil and Settings.Mode=="ImportLua" then
		PatchDataSave(Turbine.DataScope.Account,pluginName.."Story"..importWindow.currentStoryID,importStory)
	end
	--save storylist
	PatchDataSave( Turbine.DataScope.Account, pluginName.."StoryList",StoryList);
end
