--This is the main window where you choose 'Author' mode or 'Reader' mode

importPath=string.gsub(getfenv(1)._.Name,"%.Main","").."."

pluginName="Anthology"
import (importPath.."Class");
import (importPath.."VindarPatch");
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import (importPath.."Strings");
import (importPath.."Table");
import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
displayWidth=Turbine.UI.Display:GetWidth()
displayHeight=Turbine.UI.Display:GetHeight()

if (tonumber("1,000")==1) then
	function euroNormalize(value)
		return tonumber((string.gsub(value,"%.",",")));
	end
else
	function euroNormalize(value)
		return tonumber((string.gsub(value,",",".")));
	end
end

language=1;
locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end
function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end
-- load storylist so we can detect an empty list and offer to import the tutorial
StoryList=PatchDataLoad(Turbine.DataScope.Account, pluginName.."StoryList")
if StoryList==nil then StoryList={} end

-- load saved settings - we need this in Main because this supports the OptionsWindow
Settings=PatchDataLoad(Turbine.DataScope.Account, pluginName.."Settings")
if Settings==nil then Settings={} end
if Settings.LoadLastStory==nil then Settings.LoadLastStory=true end
if Settings.DataVersion==nil then
	Settings.DataVersion=plugin:GetVersion()
end
if Settings.Font==nil then
	Settings.Font=Turbine.UI.Lotro.Font.Verdana18
end
if Settings.FontColor==nil then
	Settings.FontColor=Turbine.UI.Color.White
else
	Settings.FontColor=Turbine.UI.Color(euroNormalize(Settings.FontColor.A),euroNormalize(Settings.FontColor.R),euroNormalize(Settings.FontColor.G),euroNormalize(Settings.FontColor.B))
end
if Settings.BackColor==nil then
	Settings.BackColor=Turbine.UI.Color.Black
else
	Settings.BackColor=Turbine.UI.Color(euroNormalize(Settings.BackColor.A),euroNormalize(Settings.BackColor.R),euroNormalize(Settings.BackColor.G),euroNormalize(Settings.BackColor.B))
end
if Settings.HeadingColor==nil then
	Settings.HeadingColor=Turbine.UI.Color.LightBlue
else
	Settings.HeadingColor=Turbine.UI.Color(euroNormalize(Settings.HeadingColor.A),euroNormalize(Settings.HeadingColor.R),euroNormalize(Settings.HeadingColor.G),euroNormalize(Settings.HeadingColor.B))
end
if Settings.LabelColor==nil then
	Settings.LabelColor=Turbine.UI.Color(.843,.698,.227)
else
	Settings.LabelColor=Turbine.UI.Color(euroNormalize(Settings.LabelColor.A),euroNormalize(Settings.LabelColor.R),euroNormalize(Settings.LabelColor.G),euroNormalize(Settings.LabelColor.B))
end
if Settings.TabSelectedColor==nil then
	Settings.TabSelectedColor=Turbine.UI.Color.LightBlue
else
	-- we have to euroNormalize the colors in case the calling plugin doesn't support font colors
	Settings.TabSelectedColor=Turbine.UI.Color(euroNormalize(Settings.TabSelectedColor.A),euroNormalize(Settings.TabSelectedColor.R),euroNormalize(Settings.TabSelectedColor.G),euroNormalize(Settings.TabSelectedColor.B))
end
if Settings.TabUnSelectedColor==nil then
	Settings.TabUnSelectedColor=Turbine.UI.Color(.2,.25,1)
else
	-- we have to euroNormalize the colors in case the calling plugin doesn't support font colors
	Settings.TabUnSelectedColor=Turbine.UI.Color(euroNormalize(Settings.TabUnSelectedColor.A),euroNormalize(Settings.TabUnSelectedColor.R),euroNormalize(Settings.TabUnSelectedColor.G),euroNormalize(Settings.TabUnSelectedColor.B))
end

if Settings.TrimColor==nil then
	Settings.TrimColor=Turbine.UI.Color(.15,.15,.25)
else
	Settings.TrimColor=Turbine.UI.Color(euroNormalize(Settings.TrimColor.A),euroNormalize(Settings.TrimColor.R),euroNormalize(Settings.TrimColor.G),euroNormalize(Settings.TrimColor.B))
end

if Settings.LanguageEN==nil then
	Settings.LanguageEN=(locale == "en")
end
if Settings.LanguageDE==nil then
	Settings.LanguageDE=(locale == "de")
end
if Settings.LanguageFR==nil then
	Settings.LanguageFR=(locale == "fr")
end
if Settings.Opacity==nil then
	Settings.Opacity=1
else
	Settings.Opacity=euroNormalize(Settings.Opacity)
end

if Settings.Mode=="Reader" then
	-- launch the Reader plugin then wait to be unloaded
	reloadControl=Turbine.UI.Control()
	reloadControl.Update=function()
		Turbine.PluginManager.UnloadScriptState("AnthologyImport");
		Turbine.PluginManager.UnloadScriptState("AnthologyReader");
		Turbine.PluginManager.LoadPlugin("AnthologyReader")
		reloadControl:SetWantsUpdates(false)
	end
	reloadControl:SetWantsUpdates(true)
elseif Settings.Mode=="Author" then
	-- launch the Author plugin then wait to be unloaded
	reloadControl=Turbine.UI.Control()
	reloadControl.Update=function()
		Turbine.PluginManager.UnloadScriptState("AnthologyAuthor");
		Turbine.PluginManager.LoadPlugin("AnthologyAuthor")
		reloadControl:SetWantsUpdates(false)
	end
	reloadControl:SetWantsUpdates(true)
elseif Settings.Mode=="ImportLua" or Settings.Mode=="ImportDat" or Settings.Mode=="Export" then
	-- launch the Import/Export plugin then wait to be unloaded
	reloadControl=Turbine.UI.Control()
	reloadControl.Update=function()
		Turbine.PluginManager.UnloadScriptState("AnthologyImport");
		Turbine.PluginManager.LoadPlugin("AnthologyImport")
		reloadControl:SetWantsUpdates(false)
	end
	reloadControl:SetWantsUpdates(true)
else
	-- now import
	import (importPath.."DropDownList");
	import (importPath.."RadioButtonGroup");
	import (importPath.."FontSupport")
	fontMetric=FontMetric()
	fontMetric:SetFont(Settings.Font)
	import (importPath.."DebugWindow");
	import (importPath.."PopUpDialog");
	import (importPath.."OptionsWindow");
	import (importPath.."IconWindow")
	import (importPath.."Settings")

	local buttonFont=Settings.Font
	while Turbine.UI.Lotro.FontInfo[buttonFont].size>22 do
		-- make sure button text fits on buttons since they don't size larger than 20 pixels (22 is ok since lotro font sizes are off a bit)
		buttonFont=GetSmallerFont(buttonFont)
	end

	mainWindow = Turbine.UI.Lotro.Window();
	mainWindow:SetSize(350,200);
	mainWindow:SetPosition((displayWidth-mainWindow:GetWidth())/2 ,(displayHeight-mainWindow:GetHeight())/2 );
	mainWindow:SetVisible(false);

	mainWindow.ReaderButton=Turbine.UI.Lotro.Button()
	mainWindow.ReaderButton:SetParent(mainWindow)
	mainWindow.ReaderButton:SetSize(120,20)
	mainWindow.ReaderButton:SetPosition((mainWindow:GetWidth()-mainWindow.ReaderButton:GetWidth())/2,45)
	mainWindow.ReaderButton:SetFont(buttonFont)
	mainWindow.ReaderButton:SetText(Resource[language][5])
	mainWindow.ReaderButton.Click=function()
		-- set the "ReaderCurrentStoryID=nil"
		SaveSetting("Account",pluginName.."Settings","","ReaderStoryID",nil)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Left",iconWindow:GetLeft()/displayWidth)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Top",iconWindow:GetTop()/displayHeight)
		SettingQueueCallback(launchReader)
		mainWindow:SetVisible(false)
	end

	mainWindow.AuthorButton=Turbine.UI.Lotro.Button()
	mainWindow.AuthorButton:SetParent(mainWindow)
	mainWindow.AuthorButton:SetSize(120,20)
	mainWindow.AuthorButton:SetPosition((mainWindow:GetWidth()-mainWindow.AuthorButton:GetWidth())/2,75)
	mainWindow.AuthorButton:SetFont(buttonFont)
	mainWindow.AuthorButton:SetText(Resource[language][4])
	mainWindow.Closed=function()
		iconWindow:SetVisible(true)
	end
	mainWindow.AuthorButton.Click=function()
		SaveSetting("Account",pluginName.."Settings","","AuthorStoryID",nil)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Left",iconWindow:GetLeft()/displayWidth)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Top",iconWindow:GetTop()/displayHeight)
		SettingQueueCallback(launchAuthor)
		mainWindow:SetVisible(false)
	end

	mainWindow.ImportButton=Turbine.UI.Lotro.Button()
	mainWindow.ImportButton:SetParent(mainWindow)
	mainWindow.ImportButton:SetSize(120,20)
	mainWindow.ImportButton:SetPosition((mainWindow:GetWidth()-mainWindow.ImportButton:GetWidth())/2,105)
	mainWindow.ImportButton:SetFont(buttonFont)
	mainWindow.ImportButton:SetText(Resource[language][6])
	mainWindow.ImportButton.Click=function()
		SaveSetting("Account",pluginName.."Settings","","ImportStoryID",nil)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Left",iconWindow:GetLeft()/displayWidth)
		SaveSetting("Account",pluginName.."Settings","IconWindow","Top",iconWindow:GetTop()/displayHeight)
		SettingQueueCallback(launchImport)
		mainWindow:SetVisible(false)
	end

	mainWindow.SettingsButton=Turbine.UI.Lotro.Button()
	mainWindow.SettingsButton:SetParent(mainWindow)
	mainWindow.SettingsButton:SetSize(120,20)
	mainWindow.SettingsButton:SetPosition((mainWindow:GetWidth()-mainWindow.SettingsButton:GetWidth())/2,mainWindow:GetHeight()-55)
	mainWindow.SettingsButton:SetFont(buttonFont)
	mainWindow.SettingsButton:SetText(Resource[language][52])
	mainWindow.SettingsButton.Click=function()
		OptionsWindow:SetVisible(true)
		mainWindow.MouseMask:SetMouseVisible(true)
	end

	mainWindow.MouseMask=Turbine.UI.Control()
	mainWindow.MouseMask:SetParent(mainWindow)
	mainWindow.MouseMask:SetSize(mainWindow:GetSize())
	mainWindow.MouseMask:SetMouseVisible(false)

	mainWindow:SetText(Resource[language][2]);

	mainWindow.RemoveShellCommands=function(sender,args)
		Turbine.Shell.RemoveCommand(mainWindow.shellCmd)
	end
	mainWindow.RefreshLayout=function()
		local buttonFont=Settings.Font
		while Turbine.UI.Lotro.FontInfo[buttonFont].size>22 do
			-- make sure button text fits on buttons since they don't size larger than 20 pixels (22 is ok since lotro font sizes are off a bit)
			buttonFont=GetSmallerFont(buttonFont)
		end

		fontMetric:SetFont(buttonFont)
		local minButtonWidth=120
		local tmpWidth=fontMetric:GetTextWidth(Resource[language][5])+20
		if tmpWidth>minButtonWidth then minButtonWidth=tmpWidth end
		tmpWidth=fontMetric:GetTextWidth(Resource[language][4])+20
		if tmpWidth>minButtonWidth then minButtonWidth=tmpWidth end
		tmpWidth=fontMetric:GetTextWidth(Resource[language][6])+20
		if tmpWidth>minButtonWidth then minButtonWidth=tmpWidth end
		tmpWidth=fontMetric:GetTextWidth(Resource[language][52])+20
		if tmpWidth>minButtonWidth then minButtonWidth=tmpWidth end
		if minButtonWidth>256 then minButtonWidth=256 end -- for some strange reason, skinnable buttons can only 256 pixels wide
		-- resize mainWindow if needed to accomodate large fonts
		if minButtonWidth>120 then
			mainWindow:SetWidth(minButtonWidth+230)
		else
			mainWindow:SetWidth(350)
		end
		local buttonLeft=(mainWindow:GetWidth()-minButtonWidth)/2
		mainWindow.ReaderButton:SetFont(buttonFont)
		mainWindow.ReaderButton:SetWidth(minButtonWidth)
		mainWindow.ReaderButton:SetLeft(buttonLeft)
		mainWindow.ReaderButton:SetText(Resource[language][5])
		mainWindow.AuthorButton:SetFont(buttonFont)
		mainWindow.AuthorButton:SetWidth(minButtonWidth)
		mainWindow.AuthorButton:SetLeft(buttonLeft)
		mainWindow.AuthorButton:SetText(Resource[language][4])
		mainWindow.ImportButton:SetFont(buttonFont)
		mainWindow.ImportButton:SetWidth(minButtonWidth)
		mainWindow.ImportButton:SetLeft(buttonLeft)
		mainWindow.ImportButton:SetText(Resource[language][6])
		mainWindow.SettingsButton:SetFont(buttonFont)
		mainWindow.SettingsButton:SetWidth(minButtonWidth)
		mainWindow.SettingsButton:SetLeft(buttonLeft)
		mainWindow.SettingsButton:SetText(Resource[language][52])
	end
	mainWindow.Update=function()
		pcall(Turbine.PluginManager.UnloadScriptState,"AnthologyReader");
		pcall(Turbine.PluginManager.UnloadScriptState,"AnthologyAuthor");
		pcall(Turbine.PluginManager.UnloadScriptState,"AnthologyImport");
		mainWindow:SetWantsUpdates(false)
		if #StoryList==0 then
			-- offer to auto-import tutorial
			SettingQueueCallback(mainWindow.OfferAutoImport)
		end
	end
	mainWindow.StoryDeleted=function(ID)
	end
	mainWindow:SetWantsUpdates(true)
	mainWindow.RefreshLayout()
	mainWindow.RefreshColor=function()
	end
	mainWindow.RefreshColor()
	mainWindow:SetVisible(true);
	currentWindow=mainWindow;
	mainWindow.OfferAutoImport=function()
		confirmImport=PopUpDialog(Resource[language][199],Resource[language][200],4,Resource[language][32],Resource[language][19],nil,false,mainWindow.AutoImport)
	end
	mainWindow.AutoImport=function(flag)
		if flag==1 then
			-- import the .lua file by loading the plugin
			local success,msg=pcall(Turbine.PluginManager.LoadPlugin,"AnthologyTutorial")
			if not success then
				-- generate a crit popup
				popUp=PopUpDialog(Resource[language][112],Resource[language][169]..msg,1,Resource[language][32],nil,nil,false,closePopUp)
			else
				-- _G.importStory should now be available to the AnthologyImport apartment
				SaveSetting("Account",pluginName.."Settings","","ProcessTutorial",true)
				SettingQueueCallback(launchImport)		
			end
		end
	end
end
launchReader=function()
	Turbine.PluginManager.LoadPlugin("AnthologyReader")
end
launchAuthor=function()
	Turbine.PluginManager.LoadPlugin("AnthologyAuthor")
end
launchImport=function()
	Turbine.PluginManager.LoadPlugin("AnthologyImport")
end
closePopUp=function()
	popUp=nil
end