local importPath=string.gsub(getfenv(1)._.Name,"%.Storyboard","").."."
import (importPath.."Arrow");
-- This displays the current authorStory in a "storyboad" view, graphically laid out on a large canvas with transitions as arrows

-- if a user clicks an objective, the arrow associated with that objective will get colored (blue=objNextChapter, lightgreen=trans1, lightred=trans2)
-- -- by selecting an objective and then Ctrl+clicking another chapter, the arrow will move to the coresponding chapter and update the underlying GoTo.
-- when not selected, arrows are either Blue(objGoTo), Green (trans1) or Red (trans2)
-- right clicking an objective will create a popup menu to Set Independant, Set Transition(1) or SetTransition(2)
-- -- after clicking, change color and associated NextChapter setting

if Settings.StoryboardBackColor==nil then
	Settings.StoryboardBackColor=Turbine.UI.Color(.05,.05,.05)
end
if Settings.StoryboardArrowColor==nil then
	Settings.StoryboardArrowColor=Turbine.UI.Color.White
end
if Settings.StoryboardSelectedArrowColor==nil then
	Settings.StoryboardSelectedArrowColor=Turbine.UI.Color(.3,.4,1)
end

StoryboardMenu=Turbine.UI.ContextMenu()

StoryboardMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][197]));
StoryboardMenu:GetItems():Get(1).Click=function(sender,args)
	-- create a new Chapter at mouse X, Y
	local X,Y=Turbine.UI.Display:GetMousePosition()
	local newID=#authorStory.Chapters+1
	local newChapter=ChapterIcon()
	newChapter.Index=newID
	newChapter.SetName(Resource[language][26])
	newChapter:RefreshLayout() -- will resize it to fit text
		-- position? try to set based on last known position from authorStory.Chapters
	newX=X
	newY=Y
	newChapter:SetPosition(X, Y)
	table.insert(Storyboard.Chapters,newChapter)
	local maxX,maxY=Storyboard.Canvas:GetSize()
	if X+newChapter:GetWidth()+50>maxX then maxX=X+newChapter:GetWidth()+50 end
	if Y+newChapter:GetHeight()>maxY then maxY=Y+newChapter:GetHeight() end
	if maxX>Storyboard.Canvas:GetWidth() or maxY>Storyboard.Canvas:GetHeight() then
		Storyboard.Canvas:SetSize(maxX,maxY)
	end

	-- add to authorStory
	local newChapter={}
	newChapter.DisplayOrder=newID
	newChapter.Name=Resource[language][26]
	newChapter.X=X
	newChapter.Y=Y
	newChapter.Status=false
	newChapter.Content={}
	newChapter.Objectives={}
	table.insert(authorStory.Chapters,newChapter)
	-- add to authorWindow
	local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
	local tmpItem=Turbine.UI.Label()
	tmpItem.ID=newID -- id/index and display order are same since we add to the end of list
	tmpItem:SetParent(authorWindow.ChapterList)
	tmpItem:SetSize(authorWindow.ChapterList:GetWidth(),controlHeight)
	tmpItem:SetFont(Settings.Font)
	tmpItem:SetForeColor(Settings.FontColor)
	tmpItem:SetText(Resource[language][26])
	tmpItem.Index=newID
	tmpItem.MouseDoubleClick=function(sender,args)
		authorWindow.ChapterList:SetSelectedIndex(sender.Index)
		authorWindow.ChapterEditButton.Click()
	end
	authorWindow.ChapterList:AddItem(tmpItem)
	-- update chapterMaint goto lists
	chapterMaint.ObjectiveNextChapter:AddItem(Resource[language][26],newID)
	chapterMaint.SuccessNextChapter:AddItem(Resource[language][26],newID)
	chapterMaint.FailNextChapter:AddItem(Resource[language][26],newID)
end

ChapterIcon=class(Turbine.UI.Label)
function ChapterIcon:Constructor()
	Turbine.UI.Label.Constructor(self)
	self:SetParent(Storyboard.Canvas)
	self:SetBackColor(Settings.TrimColor)
	self.Name=Turbine.UI.TextBox()
	self.Name:SetParent(self)
	self.Name:SetBackColor(Settings.BackColor)
	self.Name:SetMultiline(false)
	self.Name:SetMouseVisible(false)
	self.Name:SetPosition(1,1)
	self.Name.KeyDown=function(sender,args)
		if args.Action==75 then
			self.Name.TextChanged()
		end
	end
	self.Name.TextChanged=function()
		if self.Name:HasFocus() then
			updateChapterName(self.Index,self.Name:GetText())
		end
	end
	self.Objectives=Turbine.UI.ListBox()
	self.Objectives:SetParent(self)
	self.Objectives:SetBackColor(Settings.BackColor)
	self.Objectives:SetLeft(1)
	self.CalcWidth=function()
		local width=0
		fontMetric:SetFont(Settings.Font)
		local tmp=fontMetric:GetTextWidth(self.Name:GetText())+20
		if tmp>width then
			width=tmp
		end
		for k=1,self.Objectives:GetItemCount() do
			tmp=fontMetric:GetTextWidth(self.Objectives:GetItem(k):GetText())+20
			if tmp>width then
				width=tmp
			end
		end
		return width
	end
	self.AddObjectiveButton=Turbine.UI.Label()
	self.AddObjectiveButton:SetParent(self)
	self.AddObjectiveButton:SetLeft(1)
	self.AddObjectiveButton:SetBackColor(Turbine.UI.Color(.05,.05,.1))
--	Settings.StoryboardBackColor=Turbine.UI.Color(.05,.05,.05)
	self.AddObjectiveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.AddObjectiveButton.MouseClick=function(sender,args)
		local new=self.AddObjective(Resource[language][26],nil,#authorStory.Chapters[self.Index].Objectives+1)
		-- add the objective to authorStory.Chapter[]
		local newObj={}
		newObj.Name=Resource[language][26]
		newObj.DisplayOrder=new.Index -- always add to end of list
		authorStory.Chapters[self.Index].Objectives[new.Index]=newObj
		authorWindow.SetChangedState(true)
	end
	self.RemoveObjectiveButton=Turbine.UI.Label()
	self.RemoveObjectiveButton:SetParent(self)
	self.RemoveObjectiveButton:SetForeColor(Turbine.UI.Color.Red)
	self.RemoveObjectiveButton:SetMouseVisible(false)
	self.RemoveObjectiveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
	self.RemoveObjectiveButton:SetText("")
	self.RemoveObjectiveButton.MouseClick=function(sender,args)
		local ID=Storyboard.LastSelected.ID
		local obj=Storyboard.LastSelected
		local parent=obj:GetParent()
		-- remove Transition
		if obj.Transition~=nil then
			obj.Transition:SetVisible(false)
			obj.Transition=nil
		end
		-- remove from self.Objectives
		parent:RemoveItem(obj)
		-- remove from authorStory
		local displayOrder=authorStory.Chapters[self.Index].Objectives[ID].DisplayOrder
		table.remove(authorStory.Chapters[self.Index].Objectives,ID)
		for k,v in ipairs(authorStory.Chapters[self.Index].Objectives) do
			if v.DisplayOrder>displayOrder then v.DisplayOrder=v.DisplayOrder-1 end
		end
		if authorStory.Chapters[self.Index].SuccessCondition~=nil then
			local remove
			for k,v in ipairs(authorStory.Chapters[self.Index].SuccessCondition) do
				if v==ID then remove=k end
				if v>ID then authorStory.Chapters[self.Index].SuccessCondition[k]=v-1 end
			end
			if remove~=nil then table.remove(authorStory.Chapters[self.Index].SuccessCondition,remove) end
		end
		if authorStory.Chapters[self.Index].FailCondition~=nil then
			local remove
			for k,v in ipairs(authorStory.Chapters[self.Index].FailCondition) do
				if v==ID then remove=k end
				if v>ID then authorStory.Chapters[self.Index].FailCondition[k]=v-1 end
			end
			if remove~=nil then table.remove(authorStory.Chapters[self.Index].FailCondition,remove) end
		end
		self:SetWidth(self.CalcWidth())
		self.RefreshLayout()
		self.RefreshTransitions()

--		Storyboard.LastSelected:SetForeColor(Turbine.UI.Color.Red)
		Storyboard.LastSelected=nil
	end

	self.Transitions={} -- collection of arrow controls
	self.Refreshing=false
	self.SetName=function(strName)
		self.Name:SetForeColor(Settings.LabelColor)
		self.Name:SetFont(Settings.Font)
		self.Name:SetText(strName)
		self:SetWidth(self.CalcWidth())
		self.RefreshLayout()
		self.RefreshTransitions()
	end
	self.AddObjective=function(strName,nextChapter,ID)
		local newObjective=Turbine.UI.TextBox()
		newObjective:SetParent(self.Objectives)
		newObjective.ID=ID
		newObjective:SetMultiline(false)
		local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
		newObjective.NextChapter=nextChapter -- will be nil if "Use Transition" is set
		newObjective:SetParent(self.Objectives)
		newObjective:SetSize(width,controlHeight)
		newObjective:SetFont(Settings.Font)
		newObjective:SetText(strName)
		newObjective:SetForeColor(Settings.FontColor)
		newObjective.MouseClick=function(sender,args)
			if Storyboard.LastSelected~=nil then
				Storyboard.LastSelected:SetBackColor(nil)
				Storyboard.LastSelected:GetParent():GetParent().RemoveObjectiveButton:SetText("")
				Storyboard.LastSelected:GetParent():GetParent().RemoveObjectiveButton:SetMouseVisible(false)
				Storyboard.LastSelected:SetForeColor(Settings.FontColor)
				if Storyboard.LastSelected.Transition~=nil then
					Storyboard.LastSelected.Transition:SetBackColor(Settings.StoryboardArrowColor)
				end
			end
			Storyboard.LastSelected=sender
			Storyboard.LastSelected:SetBackColor(Turbine.UI.Color.DarkBlue)
			Storyboard.LastSelected:SetForeColor(Turbine.UI.Color.LightBlue)
				Storyboard.LastSelected:GetParent():GetParent().RemoveObjectiveButton:SetText("X")
			Storyboard.LastSelected:GetParent():GetParent().RemoveObjectiveButton:SetMouseVisible(true)
			if Storyboard.LastSelected.Transition~=nil then
				Storyboard.LastSelected.Transition:SetBackColor(Settings.StoryboardSelectedArrowColor)
			end
			
		end

		newObjective.KeyDown=function(sender,args)
			if args.Action==75 then
				newObjective.TextChanged()
			end
		end
		newObjective.TextChanged=function(sender,args)
			if newObjective:HasFocus() then
				updateObjectiveName(newObjective:GetParent():GetParent().Index,newObjective.Index,newObjective:GetText())
			end
		end
		newObjective.Index=self.Objectives:GetItemCount()+1
		self.Objectives:AddItem(newObjective)
		self:SetWidth(self.CalcWidth())
		self.RefreshLayout()
		self.RefreshTransitions()
		return newObjective
	end
	self.SizeChanged=function(sender,args)
		if not self.Refreshing then
			self.RefreshLayout()
		end
	end
	self.RefreshLayout=function()
		self.Refreshing=true
		local controlHeight=Turbine.UI.Lotro.FontInfo[Settings.Font].size+4
		local width=self:GetWidth()-2
		self.Name:SetSize(width,controlHeight)
		self.Objectives:SetTop(self.Name:GetTop()+controlHeight+1)
		self.Objectives:SetSize(width,controlHeight*self.Objectives:GetItemCount())
		self.AddObjectiveButton:SetSize(width-controlHeight-1,controlHeight)
		self.AddObjectiveButton:SetTop(self.Objectives:GetTop()+self.Objectives:GetHeight()+1)
		self.AddObjectiveButton:SetFont(Settings.Font)
		self.AddObjectiveButton:SetText("+")

		self.RemoveObjectiveButton:SetSize(controlHeight,controlHeight)
		self.RemoveObjectiveButton:SetPosition(width-controlHeight+1,self.Objectives:GetTop()+self.Objectives:GetHeight()+1)
		self.RemoveObjectiveButton:SetFont(Settings.Font)
		self.RemoveObjectiveButton:SetText(self.RemoveObjectiveButton:GetText())

		self:SetHeight(self.AddObjectiveButton:GetTop()+self.AddObjectiveButton:GetHeight()+1)
		for k=1,self.Objectives:GetItemCount() do
			self.Objectives:GetItem(k):SetWidth(width)
		end
		self.Refreshing=false
	end
	self.RefreshColor=function()
		self:SetBackColor(Settings.TrimColor)
		self.Name:SetBackColor(Settings.BackColor)
		self.Objectives:SetBackColor(Settings.BackColor)
		self.Name:SetForeColor(Settings.LabelColor)
		for k=1,self.Objectives:GetItemCount() do
			self.Objectives:GetItem(k):SetForeColor(Settings.FontColor)
		end
	end
	self.MouseDown=function(sender,args)
		if self:IsControlKeyDown() then
			self.Moving=true
			self.MoveX=args.X
			self.MoveY=args.Y
		end
	end
	self.MouseClick=function(sender,args)
		if not self.Moving then
			if self:IsShiftKeyDown() and Storyboard.LastSelected~=nil then
				authorWindow.SetChangedState(true)
				-- move the transition to here
				if Storyboard.LastSelected.Transition~=nil then
					Storyboard.LastSelected.Transition:SetVisible(false)
					Storyboard.LastSelected.Transition=nil
				end
				--now create a new transition to this chapter
				local source=Storyboard.LastSelected:GetParent():GetParent()
				Storyboard.LastSelected.Transition=Storyboard.ShowTransition(Storyboard.LastSelected:GetParent():GetParent(),self)
				Storyboard.LastSelected.Transition:SetBackColor(Settings.StoryboardSelectedArrowColor)
				Storyboard.LastSelected.NextChapter=self.Index
				-- update authorStory.Chapters[source.Index].Objectives[Storyboard.LastSelected.Index].NextChapter
				authorStory.Chapters[source.Index].Objectives[Storyboard.LastSelected.Index].NextChapter=self.Index
			else
				self.Name:Focus()
			end
		end
	end
	self.MouseUp=function(sender,args)
		if self.Moving then
			self.Moving=false
			authorStory.Chapters[self.Index].X=self:GetLeft()
			authorStory.Chapters[self.Index].Y=self:GetTop()
			self.RefreshTransitions()
		end
	end
	self.MouseMove=function(sender,args)
		if self.Moving then
			authorWindow.SetChangedState(true)
			local newLeft=self:GetLeft()-(self.MoveX-args.X)
			local newTop=self:GetTop()-(self.MoveY-args.Y)
			if newLeft<0 then newLeft=0 end;
			if newLeft+self:GetWidth()>Storyboard.Canvas:GetWidth() then Storyboard.Canvas:SetWidth(newLeft+self:GetWidth()) end;
			if newTop<0 then newTop=0 end;
			if newTop+self:GetHeight()>Storyboard.Canvas:GetHeight() then Storyboard.Canvas:SetHeight(newTop+self:GetHeight()) end
			self:SetPosition(newLeft,newTop);
			self.RefreshTransitions()
		end
	end
	self.MouseDoubleClick=function()
		Storyboard.MouseMask:SetMouseVisible(true) -- make chapterMaint modal to ease update issues
		chapterMaint.SetChapter(self.Index)
		chapterMaint:SetVisible(true)
	end
	self.RefreshTransitions=function()
		for k,v in ipairs(Storyboard.Chapters) do
			if k==self.Index then
				-- refresh outgoing objective transitions
				for index=1,v.Objectives:GetItemCount() do
					local tmpObj=v.Objectives:GetItem(index)
					local tmpColor=Settings.StoryboardArrowColor
					if tmpObj.Transition~=nil then
						-- clear current line
						tmpColor=tmpObj.Transition:GetBackColor()
						tmpObj.Transition:SetVisible(false)
						tmpObj.Transition=nil
					end
					if tmpObj.NextChapter~=nil and tmpObj.NextChapter>0 then
						tmpObj.Transition=Storyboard.ShowTransition(v,Storyboard.Chapters[tmpObj.NextChapter])
						tmpObj.Transition:SetBackColor(tmpColor)
					end
				end
				-- refresh outgoing chapter transitions
				local tmpColor=Turbine.UI.Color.Green
				if v.Success~=nil then
					-- clear current line
					tmpColor=v.Success:GetBackColor()
					v.Success:SetVisible(false)
					v.Success=nil
				end
				if v.SuccessNextChapter~=nil and v.SuccessNextChapter>0 then
					v.Success=Storyboard.ShowTransition(v,Storyboard.Chapters[v.SuccessNextChapter])
					v.Success:SetBackColor(tmpColor)
				end
				local tmpColor=Turbine.UI.Color.Red
				if v.Fail~=nil then
					-- clear current line
					tmpColor=v.Fail:GetBackColor()
					v.Fail:SetVisible(false)
					v.Fail=nil
				end
				if v.FailNextChapter~=nil and v.FailNextChapter>0 then
					v.Fail=Storyboard.ShowTransition(v,Storyboard.Chapters[v.FailNextChapter])
					v.Fail:SetBackColor(tmpColor)
				end
			else
				for index=1,v.Objectives:GetItemCount() do
					-- refresh incomming object transitions
					local tmpObj=v.Objectives:GetItem(index)
					if tmpObj.NextChapter==self.Index then
						local tmpColor=Settings.StoryboardArrowColor
						if tmpObj.Transition~=nil then
							-- clear current line
							tmpColor=tmpObj.Transition:GetBackColor()
							tmpObj.Transition:SetVisible(false)
							tmpObj.Transition=nil
						end
						if tmpObj.NextChapter~=nil and tmpObj.NextChapter>0 then
							tmpObj.Transition=Storyboard.ShowTransition(v,Storyboard.Chapters[tmpObj.NextChapter])
							if tmpObj.Transition~=nil then
								tmpObj.Transition:SetBackColor(tmpColor)
							end
						end
					end
					-- refresh incomming chapter transitions
					if v.SuccessNextChapter==self.Index then
						local tmpColor=Turbine.UI.Color.Green
						if v.Success~=nil then
							-- clear current line
							tmpColor=v.Success:GetBackColor()
							v.Success:SetVisible(false)
							v.Success=nil
						end
						v.Success=Storyboard.ShowTransition(v,Storyboard.Chapters[v.SuccessNextChapter])
						v.Success:SetBackColor(tmpColor)
					end
					if v.FailNextChapter==self.Index then
						local tmpColor=Turbine.UI.Color.Red
						if v.Fail~=nil then
							-- clear current line
							tmpColor=v.Fail:GetBackColor()
							v.Fail:SetVisible(false)
							v.Fail=nil
						end
						v.Fail=Storyboard.ShowTransition(v,Storyboard.Chapters[v.FailNextChapter])
						v.Fail:SetBackColor(tmpColor)
					end
				end
			end
		end
	end
end

Storyboard=Turbine.UI.Window()
Storyboard.ViewPane=Turbine.UI.ListBox()
Storyboard.ViewPane:SetParent(Storyboard)
Storyboard.ViewPane:SetPosition(0,20)
--Storyboard:SetResizable(true)
Storyboard:SetSize(displayWidth,displayHeight)
Storyboard.ViewPane:SetSize(Storyboard:GetWidth()-10,Storyboard:GetHeight()-30)
Storyboard:SetBackColor(Settings.BackColor)
Storyboard:SetZOrder(1)
Storyboard.CloseButton=Turbine.UI.Control()
Storyboard.CloseButton:SetParent(Storyboard)
Storyboard.CloseButton:SetSize(16,16)
Storyboard.CloseButton:SetPosition(displayWidth-17,1)

if localPlayer:GetAlignment()==2 then
	Storyboard.CloseButton:SetBackground(0x41005f41)

	Storyboard.CloseButton.UpIcon=0x41005f41
	Storyboard.CloseButton.MouseEnter=function()
		Storyboard.CloseButton:SetBackground(0x41005f50)
		Storyboard.CloseButton.UpIcon=0x41005f50
	end
	Storyboard.CloseButton.MouseDown=function()
		Storyboard.CloseButton:SetBackground(0x41005f49)
	end
	Storyboard.CloseButton.MouseLeave=function()
		Storyboard.CloseButton:SetBackground(0x41005f41)
		Storyboard.CloseButton.UpIcon=0x41005f41
	end
else
	Storyboard.CloseButton:SetBackground(0x41000196)
	Storyboard.CloseButton.UpIcon=0x41000196
	Storyboard.CloseButton.MouseEnter=function()
		Storyboard.CloseButton:SetBackground(0x41000198)
		Storyboard.CloseButton.UpIcon=0x41000198
	end
	Storyboard.CloseButton.MouseDown=function()
		Storyboard.CloseButton:SetBackground(0x41000197)
	end
	Storyboard.CloseButton.MouseLeave=function()
		Storyboard.CloseButton:SetBackground(0x41000196)
		Storyboard.CloseButton.UpIcon=0x41000196
	end
end
Storyboard.CloseButton.MouseUp=function()
	Storyboard.CloseButton:SetBackground(Storyboard.CloseButton.UpIcon)
end
Storyboard.CloseButton.MouseClick=function()
	Storyboard:SetVisible(false)
end

Storyboard.Chapters={} -- array of displays for chapters
Storyboard.Canvas=Turbine.UI.Control()
Storyboard.Canvas:SetParent(Storyboard.ViewPane)
Storyboard.Canvas:SetSize(Storyboard.ViewPane:GetWidth(),Storyboard.ViewPane:GetHeight())
Storyboard.Canvas:SetBackColor(Settings.BackColor)
Storyboard.ViewPane:SetBackColor(Settings.StoryboardBackColor)
Storyboard.Canvas:SetBackColor(Settings.StoryboardBackColor)
Storyboard.ViewPane:AddItem(Storyboard.Canvas)

Storyboard.HScroll=Turbine.UI.Lotro.ScrollBar()
Storyboard.HScroll:SetZOrder(1)
Storyboard.HScroll:SetParent(Storyboard)
Storyboard.HScroll:SetOrientation(Turbine.UI.Orientation.Horizontal)
Storyboard.HScroll:SetSize(Storyboard.ViewPane:GetWidth(),10)
Storyboard.HScroll:SetPosition(0,Storyboard.ViewPane:GetTop()+Storyboard.ViewPane:GetHeight())
Storyboard.ViewPane:SetHorizontalScrollBar(Storyboard.HScroll)

Storyboard.VScroll=Turbine.UI.Lotro.ScrollBar()
Storyboard.VScroll:SetZOrder(1)
Storyboard.VScroll:SetParent(Storyboard)
Storyboard.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
Storyboard.VScroll:SetSize(10,Storyboard.ViewPane:GetHeight())
Storyboard.VScroll:SetPosition(Storyboard.ViewPane:GetWidth(),Storyboard.ViewPane:GetTop())
Storyboard.ViewPane:SetVerticalScrollBar(Storyboard.VScroll)

Storyboard.MouseMask=Turbine.UI.Control()
Storyboard.MouseMask:SetParent(Storyboard)
Storyboard.MouseMask:SetSize(Storyboard:GetWidth(),Storyboard:GetHeight())
Storyboard.MouseMask:SetMouseVisible(false)
Storyboard.RefreshLayout=function()
	Storyboard.ViewPane:SetSize(Storyboard:GetWidth()-10,Storyboard:GetHeight()-30)
	for k,v in ipairs(Storyboard.Chapters) do
		v.RefreshLayout()
	end
end
Storyboard.Canvas.MouseClick=function(sender,args)
	if args.Button==2 then
		local x,y=Turbine.UI.Display:GetMousePosition()
		StoryboardMenu:ShowMenuAt(x,y)
	end
end

Storyboard:RefreshLayout()
Storyboard.RefreshColor=function()
	Storyboard:SetBackColor(Settings.BackColor)
	for k,v in ipairs(Storyboard.Chapters) do
		v.RefreshColor()
	end
end
Storyboard.VisibleChanged=function()
	if Storyboard:IsVisible() then
		for k,v in ipairs(Storyboard.Chapters) do
			v.RefreshTransitions()
		end
	end
end
Storyboard.SetStory=function()
	Storyboard.Canvas:SetSize(Storyboard.ViewPane:GetWidth(),Storyboard.ViewPane:GetHeight())
	if authorStory.Chapters==nil then authorStory.Chapters={} end
	if Storyboard.Chapters~=nil then
		for k,v in ipairs(Storyboard.Chapters) do
			-- clear out any handlers and objects
			for index=1,v.Objectives:GetItemCount() do
				local tmpObj=v.Objectives:GetItem(index)
				if tmpObj.Transition~=nil then
					tmpObj.Transition:SetVisible(false)
					tmpObj.Transition:SetParent(nil)
					tmpObj.Transition=nil
				end
			end
			if v.Success~=nil then
				v.Success:SetVisible(false)
				v.Success:SetParent(nil)
				v.Success=nil
			end
			if v.Fail~=nil then
				v.Fail:SetVisible(false)
				v.Fail:SetParent(nil)
				v.Fail=nil
			end
			Storyboard.Chapters[k]:SetParent(nil)
			Storyboard.Chapters[k]=nil
		end
	end
	Storyboard.Chapters={}
	-- now add all of the chapter icons
	local maxX=Storyboard.ViewPane:GetWidth()
	local maxY=Storyboard.ViewPane:GetHeight()
	local newX=0
	local newY=0
	for k,v in ipairs(authorStory.Chapters) do
		local newChapter=ChapterIcon()
		newChapter.Index=k
		newChapter.SetName(v.Name)
		-- add all of the objective names
		if v.Objectives~=nil then
			for index,obj in ipairs(v.Objectives) do
				newChapter.AddObjective(obj.Name,obj.NextChapter,index)
			end
		end
		newChapter:RefreshLayout() -- will resize it to fit text
		-- position? try to set based on last known position from authorStory.Chapters
		if v.X==nil or v.Y==nil then
			-- calculate a free space, at least 50 pixels from nearest current chapter
			v.X=newX+200
			v.Y=newY
			if v.X>displayWidth then
				v.X=0
				v.Y=v.Y+200
			end
		end
		newX=v.X
		newY=v.Y
		newChapter:SetPosition(v.X, v.Y)
		table.insert(Storyboard.Chapters,newChapter)
		if v.X+newChapter:GetWidth()+50>maxX then maxX=v.X+newChapter:GetWidth()+50 end
		if v.Y+newChapter:GetHeight()>maxY then maxY=v.Y+newChapter:GetHeight() end
	end
	if maxX>Storyboard.Canvas:GetWidth() or maxY>Storyboard.Canvas:GetHeight() then
		Storyboard.Canvas:SetSize(maxX,maxY)
	end

	-- now that all of the chapter controls are created, loop back through chapters, adding arrows for each transition
	for k,v in ipairs(Storyboard.Chapters) do
		for index=1,v.Objectives:GetItemCount() do
			local tmpObj=v.Objectives:GetItem(index)
			if tmpObj.Transition~=nil then
				-- clear current line
				tmpObj.Transition:SetVisible(false)
				tmpObj.Transition=nil
			end
			if tmpObj.NextChapter~=nil then
				tmpObj.Transition=Storyboard.ShowTransition(v,Storyboard.Chapters[tmpObj.NextChapter])
			end
		end
		if authorStory.Chapters[v.Index].SuccessNextChapter==nil or authorStory.Chapters[v.Index].SuccessNextChapter<1 then
			v.Success=nil
			v.SuccessNextChapter=nil
		else
			v.Success=Storyboard.ShowTransition(v,Storyboard.Chapters[authorStory.Chapters[v.Index].SuccessNextChapter])
			v.Success:SetBackColor(Turbine.UI.Color.Green)
			v.SuccessNextChapter=authorStory.Chapters[v.Index].SuccessNextChapter
		end
		if authorStory.Chapters[v.Index].FailNextChapter==nil or authorStory.Chapters[v.Index].FailNextChapter<1 then
			v.Fail=nil
			v.FailNextChapter=nil
		else
			v.Fail=Storyboard.ShowTransition(v,Storyboard.Chapters[authorStory.Chapters[v.Index].FailNextChapter])
			v.Fail:SetBackColor(Turbine.UI.Color.Red)
			v.FailNextChapter=authorStory.Chapters[v.Index].FailNextChapter
		end
	end
end
Storyboard.ShowTransition=function(source, dest)
	local arrow
	-- determine center of each
	if source~=nil and dest~=nil then
		sourceCenterX=source:GetLeft()+source:GetWidth()/2
		sourceCenterY=source:GetTop()+source:GetHeight()/2
		destCenterX=dest:GetLeft()+dest:GetWidth()/2
		destCenterY=dest:GetTop()+dest:GetHeight()/2
	-- find angle between centers
	 	angle=math.atan((destCenterY-sourceCenterY)/(destCenterX-sourceCenterX))
		sourceAngle=math.atan(source:GetHeight()/source:GetWidth())
		destAngle=math.atan(dest:GetHeight()/dest:GetWidth())
	-- determine where the line intersects the border of each - need angle of center to corner of source & dest

		if sourceCenterX<destCenterX then
			if angle<0 then
				angle=0-angle
			-- quadrant 1 sourceCenterX<destCenterX
				if angle<sourceAngle then
					sourceX=sourceCenterX+source:GetWidth()/2
					sourceY=sourceCenterY-math.tan(angle)*source:GetWidth()/2
				else
					sourceY=sourceCenterY-source:GetHeight()/2
					sourceX=sourceCenterX+math.tan(math.pi/2-angle)*source:GetHeight()/2
				end
				if angle<destAngle then
					destX=destCenterX-dest:GetWidth()/2
					destY=destCenterY+math.tan(angle)*dest:GetWidth()/2
				else
					destY=destCenterY+dest:GetHeight()/2
					destX=destCenterX-math.tan(math.pi/2-angle)*dest:GetHeight()/2
				end
			else
				-- quadrant 4
				if angle<sourceAngle then
					sourceX=sourceCenterX+source:GetWidth()/2
					sourceY=sourceCenterY+math.tan(angle)*source:GetWidth()/2
				else
					sourceY=sourceCenterY+source:GetHeight()/2
					sourceX=sourceCenterX+math.tan(math.pi/2-angle)*source:GetHeight()/2
				end
				if angle<destAngle then
					destX=destCenterX-dest:GetWidth()/2
					destY=destCenterY-math.tan(angle)*dest:GetWidth()/2
				else
					destY=destCenterY-dest:GetHeight()/2
					destX=destCenterX-math.tan(math.pi/2-angle)*dest:GetHeight()/2
				end
			end
		else
			if angle<0 then
				-- quadrant 3
				angle=0-angle
				if angle<sourceAngle then
					sourceX=sourceCenterX-source:GetWidth()/2
					sourceY=sourceCenterY+math.tan(angle)*source:GetWidth()/2
				else
					sourceY=sourceCenterY+source:GetHeight()/2
					sourceX=sourceCenterX-math.tan(math.pi/2-angle)*source:GetHeight()/2
				end
				if angle<destAngle then
					destX=destCenterX+dest:GetWidth()/2
					destY=destCenterY-math.tan(angle)*dest:GetWidth()/2
				else
					destY=destCenterY-dest:GetHeight()/2
					destX=destCenterX+math.tan(math.pi/2-angle)*dest:GetHeight()/2
				end
			else
				-- quadrant 2
				if angle<sourceAngle then
					sourceX=sourceCenterX-source:GetWidth()/2
					sourceY=sourceCenterY-math.tan(angle)*source:GetWidth()/2
				else
					sourceY=sourceCenterY-source:GetHeight()/2
					sourceX=sourceCenterX-math.tan(math.pi/2-angle)*source:GetHeight()/2
				end
				if angle<destAngle then
					destX=destCenterX+dest:GetWidth()/2
					destY=destCenterY+math.tan(angle)*dest:GetWidth()/2
				else
					destY=destCenterY+dest:GetHeight()/2
					destX=destCenterX+math.tan(math.pi/2-angle)*dest:GetHeight()/2
				end
			end
		end
		-- draw arrow between intersects
		arrow=Arrow(sourceX,sourceY,destX,destY,3,7,Settings.StoryboardArrowColor,1,-1)
		arrow:SetParent(Storyboard.Canvas)
		arrow:SetMouseVisible(false) -- the lines go outside their parent and can block other items
	end
	return arrow
end