-- This plugin helps provide real-time access to .plugindata files
-- This plugin gets it's parameters from a series of commands registered by the requesting plugin
-- "0SSS_scope" where "scope" is one of three scopes, "Account", "Server", and "Character" i.e. either 0SSS_Account, 0SSS_Server or 0SSS_Character. if 0SSS is not defined it defaults to Character
-- "0SSF_file" where "file" is the file name of the .plugindata file
-- "0SSB_setting" where "setting" is the branch name of the data being saved - that is ABC[1][2][x] would pass "ABC_1_2" -- note, if the branch contains an underscore it must be encoded as /_ if it contains a / it must be encoded as //
-- "0SSKx_" is the array of Key names -- note, 0SSK0_ is used to retrieve a specific value, not a whole branch
-- "0SST" is a flag that when present indicates that, in "Save" mode, subtable entries should be wiped as well
-- "0SSVx_y_value" is the array of Values where the second parameter is for multi-chunk strings (strings are broken into 115800 character chunks to fit into the 115825 character max size for commands leaving 25 characters for the prefix, index and part components
-- if there are additional parts of the value (each string part has a max character count) they are 0SSV2, 0SSV3, etc.
-- "0SSM_mode" where mode is either "Save" or "Load"
-- if 0SSM_mode=0SSM_Save then the values in 0SSV commands are combined and saved
-- is 0SSM_mode=0SSM_Load then the value of the settings is returned as a string in 0SSR_value commands of if a table, the keys are returned as a series of 0SSZx_value commands
-- once  done, it creates the 0SSC command which tells the calling plugin that it is complete and data is saved or available for parsing. The calling plugin plugin then terminates this plugin
-- on unload, the 0SSC command and any 0SSR commands are removed.
-- "0SSD" is a flag that when present enables debug messages

-- note, to avoid conflicts, the calling plugin must check for the "0SSL" semaphore command BEFORE creating their first command and must immediately create their own "0SSL" command to get a lock on the plugin
-- the calling plugin must then clean up all of the other commands that it created

local importPath=string.gsub(getfenv(1)._.Name,"%.Main","").."."

import (importPath.."Class"); -- only needed for debugging
import (importPath.."Table"); -- only needed for debugging
import (importPath.."VindarPatch");

import "Turbine";

respCmd={}
replaceStr={[128]="A",[129]="A",[130]="A",[131]="A",[132]="A",[133]="A",[134]="AE",[135]="C",[136]="E",[137]="E",[138]="E",[139]="E",[140]="I",[141]="I",[142]="I",[143]="I",[144]="",[145]="N",[146]="O",[147]="O",[148]="O",[149]="O",[150]="O",[151]="",[152]="O",[153]="U",[154]="U",[155]="U",[156]="U",[157]="Y",[158]="Y",[159]="sz",[160]="a",[161]="a",[162]="a",[163]="a",[164]="a",[165]="a",[166]="ae",[167]="c",[168]="e",[169]="e",[170]="e",[171]="e",[172]="i",[173]="i",[174]="i",[175]="i",[176]="o",[177]="n",[178]="o",[179]="o",[180]="o",[181]="o",[182]="o",[183]="",[184]="o",[185]="u",[186]="u",[187]="u",[188]="u",[189]="y",[190]="y",[191]="y"}
string.stripaccent=function(str)
	local ret = "";
	if str~=nil then
		ret=""
		local replace=false
		for i,v in ipairs({str:byte(1,-1)}) do
			if replace then
				replace=false
				if replaceStr[v]~=nil then
					ret=ret..replaceStr[v]
				end
			else
				if (v==195) then
					replace=true
				else
					ret = ret .. string.char(v);
				end
			end
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
string.encode=function(str)
	-- replace all non-alpha characters with a % escaped hex code
	local ret
	if str~=nil then
		if type(str)=="number" then
			ret="#"..tostring(str)
		elseif type(str)=="boolean" then
			ret="$"..tostring(str)
		elseif type(str)=="string" then
			ret=""
			for i,v in ipairs({str:byte(1,-1)}) do
				if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
					ret = ret .. "%" .. string.sub("00"..string.format("%x",tonumber(v)),-2)
				else
					ret = ret .. string.char(v)
				end
			end
		elseif type(str)=="table" then
			-- return list of keys
			ret="^"
			for k,v in pairs(str) do
				if type(k)=="number" then
					ret=ret.."|#"..tostring(k)
				else
					ret=ret.."|"..string.encode(k)
				end
			end
		else
			-- what do we do with values that aren't any of the expected types?
Turbine.Shell.WriteLine("unexpected type in string.encode:"..tostring(type(str)))
			ret=tostring(val)
		end
	end
	return ret
end
string.decode=function(str)
	local ret
	if str~=nil and type(str)=="string" then
		if string.sub(str,1,1)=="#" then
			ret=tonumber(string.sub(str,2))
		elseif string.sub(str,1,1)=="$" then
			ret=(string.sub(str,2)=="true")
		elseif string.sub(str,1,1)=="^" then
			ret=string.split(string.sub(str,3),"|")
			for k,v in pairs(ret) do
				if string.sub(v,1,1)=="#" then
					ret[k]=tonumber(string.sub(v,2))
				else
					ret[k]=string.decode(v)
				end
			end
		else
			ret=""
			while string.len(str)~=0 do
				local char=string.sub(str,1,1)
				if char~="%" then
					ret=ret..char
					str=string.sub(str,2,-1)
				else
					local char=string.char(tonumber("0x"..string.sub(str,2,3)))
					ret=ret..char
					str = string.sub(str,4,-1)
				end
			end
		end
	end
	return ret;
end

local tmpScope="Character"
local file=""
local mode="Load" -- default to a non-destructive mode
local dataType="S"
local branch={}
local key={}
local value={}
local exec={}
local processSubTables=false
local debug=false

local cmds=Turbine.Shell.GetCommands();
if cmds~=nil and type(cmds)=="table" then
	local tmp,tmpIndex,tmpPart
	table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
	for cmdIndex=0,#cmds do
		if cmds[cmdIndex]~=nil then
			if cmdIndex>1 and cmds[cmdIndex]>="1" then
				break
			else
				local cmdPrefix=string.sub(cmds[cmdIndex],1,4)
				if cmdPrefix=="0SST" then
					processSubTables=true
				elseif cmdPrefix=="0SSS" then
					tmp=string.match(cmds[cmdIndex],"0SSS_(.*)");
					if tmp~=nil then tmpScope=tmp end
				elseif cmdPrefix=="0SSF" then
					tmp=string.match(cmds[cmdIndex],"0SSF_(.*)");
					if tmp~=nil then file=tmp end
				elseif cmdPrefix=="0SSK" then
					tmpIndex,tmp=string.match(cmds[cmdIndex],"0SSK(.*)_(.*)");
					if tmpIndex~=nil and tmp~=nil then
						tmpIndex=tonumber(tmpIndex)
						tmp=string.decode(tmp)
						if string.sub(tmp,1,1)=="#" then
							tmp=tonumber(string.sub(tmp,2))
						end
						key[tmpIndex]=tmp
					end
				elseif cmdPrefix=="0SSB" then
					tmp=string.match(cmds[cmdIndex],"0SSB_(.*)");
					if tmp~=nil then
						branch=string.split(tmp,"_")
						for k,v in ipairs(branch) do
							branch[k]=string.decode(v)
						end
					end
				elseif cmdPrefix=="0SSV" then
					tmpIndex,tmpPart,tmp=string.match(cmds[cmdIndex],"0SSV(.*)_(.*)_(.*)")
					if tmpIndex~=nil and tmpPart~=nil and tmp ~=nil then
						tmpIndex=tonumber(tmpIndex)
						tmpPart=tonumber(tmpPart)
						if value[tmpIndex]==nil then value[tmpIndex]={} end
						value[tmpIndex][tmpPart]=tmp
					end
				elseif cmdPrefix=="0SSD" then
					debug=true
				elseif cmdPrefix=="0SSM" then
					tmp=string.match(cmds[cmdIndex],"0SSM_(.*)");
					if tmp~=nil then mode=tmp end
				elseif cmdPrefix=="0SSE" then
					tmpPart,tmp=string.match(cmds[cmdIndex],"0SSE(.*)_(.*)")
					if tmpPart~=nil and tmp~=nil then
						tmpPart=tonumber(tmpPart)
						exec[tmpPart]=tmp
					end
				end
			end
		end
	end
end

-- reconstitute long values
for k,v in pairs(value) do
	local tmpVal=""
	for part,val in ipairs(v) do
		tmpVal=tmpVal..val
	end
	value[k]=string.decode(tmpVal)
end
command=""
for k,v in ipairs(exec) do
	command=command..v
end
command=string.decode(command)
if debug then
	Turbine.Shell.WriteLine("exec:")
	Table.Dump(exec)
	Turbine.Shell.WriteLine("command:"..tostring(command))
end

-- "0SSS_scope" where "scope" is one of three scopes, "Account", "Server", and "Character" i.e. either 0SSS_Account, 0SSS_Server or 0SSS_Character. if 0SSS is not defined it defaults to Character
-- "0SSF_file" where "file" is the file name of the .plugindata file
-- "0SSB_setting" where "setting" is the encoded branch name of the data being saved - that is ABC[1][2][x] would pass "1_2"
-- "0SSV1_value" is the first part of the string representation of the value to be saved.
-- if there are additional parts of the value (each string part has a max character count) they are 0SSV1, 0SSV2, etc.
-- "0SSM_mode" where mode is either "Save" or "Load"

-- note, if "0SSKx_setting" format is used, then 0SSK is actually an array of keys and we need to find the matching array of values



plugin.Unload = function(statusDisplay,sender,args)
	-- remove the "0SSR" commands and the "0SSC" command
	for k,v in pairs(respCmd) do
		Turbine.Shell.RemoveCommand(v)
	end
end
local success,retval;
respCmd={}
-- load the specified file (we have to load even when saving so that we can preserve any other data in this file)
local scope=Turbine.DataScope.Character
if tmpScope=="Account" then scope=Turbine.DataScope.Account end
if tmpScope=="Server" then scope=Turbine.DataScope.Server end
if debug then
	Turbine.Shell.WriteLine("scope:"..tostring(tmpScope).." file:"..tostring(file).." mode:"..tostring(mode).." branch:"..tostring(branch).." processSubTables:"..tostring(processSubTables))
	for k,v in ipairs(branch) do
		Turbine.Shell.WriteLine("Branch:"..type(v).." "..tostring(v))
	end
	Turbine.Shell.WriteLine("key(s):")
	Table.Dump(key)
	Turbine.Shell.WriteLine("value(s):")
	Table.Dump(value)
end
if file~=nil and file~="" then
	tmpData=PatchDataLoad(scope, file);
	if tmpData==nil then tmpData={} end
	local leaf=tmpData
	if mode=="Save" then
		-- follow the list of keys in setting and create any sub-tables that don't exist
		for k,v in ipairs(branch) do
if debug then
	Turbine.Shell.WriteLine("following branch:"..tostring(v))
end
			if leaf[v]==nil then
				leaf[v]={}
			end
			leaf=leaf[v]
		end
		-- leaf now points to the branch node where we are supposed to save the values
		if key[0]~=nil then
if debug then
	Turbine.Shell.WriteLine("setting leaf["..tostring(key[0]).."]="..tostring(value[0]))
end
			-- the data passed was a single value
--***
--			leaf[key[0]]=string.decode(value[0])
			leaf[key[0]]=value[0]
		else
			for k,v in pairs(leaf) do
				-- clear out old values and subtables
				if type(v)~="table" or processSubTables then
					leaf[k]=nil
				end
			end
			for k,v in ipairs(key) do
				leaf[v]=value[k]
			end
		end
		-- save the data
		local result=PatchDataSave(scope, file, tmpData);
	elseif mode=="Exec" then
		-- compile and execute the contents of command
if debug then
	Turbine.Shell.WriteLine("command:"..tostring(command))
end

		local userFunc,error=loadstring(command);
		if userFunc==nil then
			Turbine.Shell.WriteLine("Error:"..tostring(error));
		else
			setfenv(userFunc,getfenv()); -- we set the environment to the current plugin environment
			local success,commandResult=pcall(userFunc);
			if success then
				if type(commandResult)~="table" then
					local tmpData=string.encode(commandResult)
					local tmpPart=1
					while tmpData~=nil do
						newCmd=Turbine.ShellCommand()
						table.insert(respCmd,newCmd)
						Turbine.Shell.AddCommand("0SSR0_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
						tmpData=string.sub(tmpData,115801)
						if tmpData=="" then tmpData=nil end
						tmpPart=tmpPart+1
					end
				else
					local tmpIndex=1
					for k,v in pairs(commandResult) do
						local tmpData=string.encode(v)
						local tmpPart=1
						while tmpData~=nil do
							newCmd=Turbine.ShellCommand()
							table.insert(respCmd,newCmd)
							Turbine.Shell.AddCommand("0SSR"..tostring(tmpIndex).."_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
							tmpData=string.sub(tmpData,115801)
							if tmpData=="" then tmpData=nil end
							tmpPart=tmpPart+1
						end
						local newCmd=Turbine.ShellCommand()
						table.insert(respCmd,newCmd)
						Turbine.Shell.AddCommand("0SSZ"..tostring(tmpIndex).."_"..string.encode(k),newCmd)
						tmpIndex=tmpIndex+1
					end
				end
			else
				Turbine.Shell.WriteLine("Error:"..tostring(commandResult))
			end
		end
		local result=PatchDataSave(scope, file, tmpData);
	elseif mode=="Load" then
		for k,v in ipairs(branch) do
if debug then
	Turbine.Shell.WriteLine("following branch:"..tostring(v))
end
			leaf=leaf[v]
			if leaf==nil then break end
		end
		local tmpIndex=1
		if leaf==nil then
			-- nothing to return
		else
			if type(leaf)~="table" then
--*** user specified a branch but it is a leaf
						local tmpData=string.encode(leaf)
						local tmpPart=1
						while tmpData~=nil do
							newCmd=Turbine.ShellCommand()
							table.insert(respCmd,newCmd)
							Turbine.Shell.AddCommand("0SSR0_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
							tmpData=string.sub(tmpData,115801)
							if tmpData=="" then tmpData=nil end
							tmpPart=tmpPart+1
						end
						local newCmd=Turbine.ShellCommand()
						table.insert(respCmd,newCmd)
						Turbine.Shell.AddCommand("0SSZ0_"..string.encode(branch[#branch]),newCmd)
			else
				if key[0]~=nil then
if debug then
	Turbine.Shell.WriteLine("key[0]:"..tostring(key[0]))
	Turbine.Shell.WriteLine("leaf[key[0]]:"..tostring(leaf[key[0]]))
end
					if type(leaf[key[0]])=="table" then
						leaf=leaf[key[0]]
						key[0]=nil
					else
						local tmpData=string.encode(leaf[key[0]])
						local tmpPart=1
						while tmpData~=nil do
							newCmd=Turbine.ShellCommand()
							table.insert(respCmd,newCmd)
							Turbine.Shell.AddCommand("0SSR0_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
							tmpData=string.sub(tmpData,115801)
							if tmpData=="" then tmpData=nil end
							tmpPart=tmpPart+1
						end
						local newCmd=Turbine.ShellCommand()
						table.insert(respCmd,newCmd)
						Turbine.Shell.AddCommand("0SSZ0_"..string.encode(key[0]),newCmd)
					end
				end
				if key[0]==nil then
if debug then
	Turbine.Shell.WriteLine("key[0] is nil")
end
					if key[1]~=nil then
if debug then
	Turbine.Shell.WriteLine("matching user keys")
end
						-- user specified keys, try to match them
						local tmpIndex=1
						for k,v in ipairs(key) do
if debug then
	Turbine.Shell.WriteLine("key["..k.."] leaf["..tostring(v).."]:"..tostring(leaf[v]))
end
							if leaf[v]~=nil then
								local tmpData=string.encode(leaf[v])
								local tmpPart=1
								while tmpData~=nil do
									newCmd=Turbine.ShellCommand()
									table.insert(respCmd,newCmd)
									Turbine.Shell.AddCommand("0SSR"..tostring(tmpIndex).."_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
									tmpData=string.sub(tmpData,115801)
									if tmpData=="" then tmpData=nil end
									tmpPart=tmpPart+1
								end

								local newCmd=Turbine.ShellCommand()
								table.insert(respCmd,newCmd)
								Turbine.Shell.AddCommand("0SSZ"..tostring(tmpIndex).."_"..string.encode(v),newCmd)
								tmpIndex=tmpIndex+1
							end
						end
					else
						-- no list of keys, return all available values, note, subtables will return a nil value, just a key
						local tmpIndex=1
						for k,v in pairs(leaf) do
							local tmpData=string.encode(v)
							local tmpPart=1
							while tmpData~=nil do
								newCmd=Turbine.ShellCommand()
								table.insert(respCmd,newCmd)
								Turbine.Shell.AddCommand("0SSR"..tostring(tmpIndex).."_"..tostring(tmpPart).."_"..string.sub(tmpData,1,115800),newCmd)
								tmpData=string.sub(tmpData,115801)
								if tmpData=="" then tmpData=nil end
								tmpPart=tmpPart+1
							end
							local newCmd=Turbine.ShellCommand()
							table.insert(respCmd,newCmd)
							Turbine.Shell.AddCommand("0SSZ"..tostring(tmpIndex).."_"..string.encode(k),newCmd)
							tmpIndex=tmpIndex+1
						end
					end
				end
			end
		end
	end
end
local unloadCmd=Turbine.ShellCommand()
table.insert(respCmd,unloadCmd);
Turbine.Shell.AddCommand("0SSC",unloadCmd);
